import React, { useState, useEffect, useContext, useMemo, useCallback, useLayoutEffect, forwardRef, useImperativeHandle, useRef } from 'react';
import { Schema as Schema$1, DOMSerializer, Slice, DOMParser } from 'prosemirror-model';
import CryptoJS from 'crypto-js';
import stringify from 'safe-stable-stringify';
import { v4 } from 'uuid';
import { injectable, inject, multiInject, Container } from 'inversify';
import 'reflect-metadata';
import deepmerge from 'deepmerge';
import { set, get, isArrayLikeObject, omit, isPlainObject, each, eachRight, isEmpty, map, isFunction, isObject, cloneDeep } from 'lodash';
import { smartQuotes, inputRules } from 'prosemirror-inputrules';
import { keymap } from 'prosemirror-keymap';
import { undo, redo, history } from 'prosemirror-history';
import { splitListItem } from 'prosemirror-schema-list';
import { NodeSelection, TextSelection, PluginKey, Plugin, Selection, EditorState } from 'prosemirror-state';
import { chainCommands, deleteSelection, joinBackward, selectNodeBackward, selectParentNode, exitCode, baseKeymap, wrapIn } from 'prosemirror-commands';
import { replaceStep as replaceStep$1, Mapping, ReplaceStep, RemoveMarkStep, AddMarkStep, ReplaceAroundStep, liftTarget } from 'prosemirror-transform';
import ReactDOM from 'react-dom';
import styled, { css } from 'styled-components';
import { useTranslation } from 'react-i18next';
import { th, override, grid } from '@pubsweet/ui-toolkit';
import { DecorationSet, Decoration, EditorView } from 'prosemirror-view';
import { dropCursor } from 'prosemirror-dropcursor';
import { gapCursor } from 'prosemirror-gapcursor';
import moment from 'moment';
import propTypes from 'prop-types';

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(n);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

var DefaultSchema = {
  nodes: {
    doc: {
      content: 'block+'
    },
    text: {
      group: 'inline'
    },
    hard_break: {
      inline: true,
      group: 'inline',
      selectable: false,
      parseDOM: [{
        tag: 'br'
      }],
      toDOM: function toDOM() {
        return ['br'];
      }
    },
    paragraph: {
      group: 'block',
      content: 'inline*',
      attrs: {
        "class": {
          "default": 'paragraph'
        }
      },
      parseDOM: [{
        tag: 'p.paragraph',
        getAttrs: function getAttrs(dom) {
          return {
            "class": dom.getAttribute('class')
          };
        }
      }],
      toDOM: function toDOM(node) {
        return ['p', node.attrs, 0];
      }
    }
  },
  marks: {}
};

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

var ApplicationContext = React.createContext({
  app: null,
  getPlugins: null
});
var ApplicationProvider = (function (_ref) {
  var app = _ref.app,
      children = _ref.children;

  var _useState = useState(app),
      _useState2 = _slicedToArray(_useState, 2),
      application = _useState2[0],
      setApplication = _useState2[1];

  useEffect(function () {
    setApplication(_objectSpread2({}, app));
  }, [app]);
  return /*#__PURE__*/React.createElement(ApplicationContext.Provider, {
    value: {
      app: application
    }
  }, children);
});

var WaxContext = React.createContext({
  pmViews: {},
  activeView: {},
  activeViewId: null,
  updateView: null,
  updateState: null,
  updateActiveView: null,
  removeView: null
});
var WaxProvider = (function (props) {
  var _useState = useState({
    pmViews: props.view || {},
    activeView: props.activeView || {},
    activeViewId: props.activeViewId || {},
    options: {
      fullScreen: false
    },
    transaction: {}
  }),
      _useState2 = _slicedToArray(_useState, 2),
      context = _useState2[0],
      setContext = _useState2[1];

  return /*#__PURE__*/React.createElement(WaxContext.Provider, {
    value: _objectSpread2(_objectSpread2({}, context), {}, {
      updateView: function updateView(newView, activeViewId) {
        var pmViews = Object.assign(context.pmViews, newView);
        var activeView = pmViews[activeViewId || context.activeViewId];
        setContext(_objectSpread2(_objectSpread2({}, context), {}, {
          pmViews: pmViews,
          activeView: activeView,
          activeViewId: activeViewId || context.activeViewId
        }));
      },
      setTransaction: function setTransaction(tr) {
        Object.assign(context.transaction, tr);
      },
      setOption: function setOption(option) {
        Object.assign(context.options, option);
      },
      removeView: function removeView(deletedView) {
        delete context.pmViews[deletedView];
      }
    })
  }, props.children);
});
var useInjection = function useInjection(identifier) {
  var context = useContext(ApplicationProvider);

  if (!context.app.container) {
    throw new Error();
  }

  return context.app.container.isBound(identifier) ? {
    instance: context.app.container.get(identifier)
  } : null;
};

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

var PortalContext = React.createContext({
  createPortal: function createPortal() {},
  portals: {}
});
var PortalProvider = (function (props) {
  var _useState = useState({
    createPortal: function createPortal(element, component, node, view, getPos, decorations, context) {
      setTimeout(function () {
        portal.portals.push({
          id: v4(),
          element: element,
          component: component,
          node: node,
          view: view,
          getPos: getPos,
          decorations: decorations,
          context: context
        });
        setPortal({
          createPortal: portal.createPortal,
          portals: _toConsumableArray(portal.portals)
        });
      });
    },
    portals: []
  }),
      _useState2 = _slicedToArray(_useState, 2),
      portal = _useState2[0],
      setPortal = _useState2[1];

  return /*#__PURE__*/React.createElement(PortalContext.Provider, {
    value: _objectSpread2({}, portal)
  }, props.children);
});

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

var _dec, _class, _temp;
var Config = (_dec = injectable(), _dec(_class = (_temp = /*#__PURE__*/function () {
  function Config(config) {
    _classCallCheck(this, Config);

    this._config = {};
    this._config = config;
  }

  Config = inject('config')(Config, undefined, 0) || Config;

  _createClass(Config, [{
    key: "set",
    value: function set$1(key, value) {
      set(this._config, key, value);

      return this._config;
    }
  }, {
    key: "get",
    value: function get$1(key) {
      return get(this._config, key);
    }
  }, {
    key: "pushToArray",
    value: function pushToArray(key, value) {
      var oldValue = this.get(key);

      if (oldValue) {
        if (isArrayLikeObject(value)) {
          value.forEach(function (v) {
            oldValue.push(v);
          });
        } else {
          oldValue.push(value);
        }
      } else {
        oldValue = value;
      }

      this.set(key, oldValue);
      return this;
    }
  }]);

  return Config;
}(), _temp)) || _class);

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Date.prototype.toString.call(Reflect.construct(Date, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _typeof(obj) {
  "@babel/helpers - typeof";

  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function _typeof(obj) {
      return typeof obj;
    };
  } else {
    _typeof = function _typeof(obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (_typeof(call) === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

function _createSuper(Derived) {
  return function () {
    var Super = _getPrototypeOf(Derived),
        result;

    if (_isNativeReflectConstruct()) {
      var NewTarget = _getPrototypeOf(this).constructor;
      result = Reflect.construct(Super, arguments, NewTarget);
    } else {
      result = Super.apply(this, arguments);
    }

    return _possibleConstructorReturn(this, result);
  };
}

var Service = /*#__PURE__*/function () {
  function Service() {
    _classCallCheck(this, Service);
  }

  _createClass(Service, [{
    key: "setApp",
    value: function setApp(app) {
      this.app = app;
    }
  }, {
    key: "container",
    get: function get() {
      return this.app.container;
    }
  }, {
    key: "config",
    get: function get() {
      return this.app.config.get("config.".concat(this.name)) || this.app.config;
    }
  }, {
    key: "schema",
    get: function get() {
      return this.app.getSchema();
    }
  }]);

  return Service;
}();

var Middleware = /*#__PURE__*/function () {
  function Middleware() {
    _classCallCheck(this, Middleware);

    // Array prototype last
    if (!Array.prototype.last) {
      Array.prototype.last = function () {
        return this[this.length - 1];
      };
    } // Array prototype reduceOneRight


    if (!Array.prototype.reduceOneRight) {
      Array.prototype.reduceOneRight = function () {
        return this.slice(0, -1);
      };
    }
  }

  _createClass(Middleware, [{
    key: "use",
    value: function use(fn) {
      var _this = this;

      this.go = function (stack) {
        return function () {
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          return stack.apply(void 0, _toConsumableArray(args.reduceOneRight()).concat([function () {
            var _next = args.last();

            fn.apply(_this, [].concat(_toConsumableArray(args.reduceOneRight()), [_next.bind.apply(_next, [null].concat(_toConsumableArray(args.reduceOneRight())))]));
          }]));
        };
      }(this.go);
    }
  }, {
    key: "go",
    value: function go() {
      for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      var _next = args.last();

      _next.apply(this, args.reduceOneRight());
    }
  }]);

  return Middleware;
}();

var ParseRule = /*#__PURE__*/function () {
  function ParseRule(_ref) {
    var getAttrs = _ref.getAttrs,
        tag = _ref.tag,
        style = _ref.style;

    _classCallCheck(this, ParseRule);

    this.tag = null;
    this.style = null;
    this.exporter = null;

    this.defaultMiddleware = function (hook, next) {
      next();
    };

    this.tag = tag;
    this.style = style; // if (getAttrs) {

    this.exporter = new Middleware(); // }

    this.addStack(getAttrs);
  }

  _createClass(ParseRule, [{
    key: "addStack",
    value: function addStack(getAttrs) {
      if (getAttrs) {
        this.exporter.use(getAttrs);
      } else {
        this.exporter.use(this.defaultMiddleware);
      }
    }
  }, {
    key: "parseSchema",
    value: function parseSchema(exporter) {
      var rule = {};

      if (this.tag) {
        rule = {
          tag: this.tag
        };
      }

      if (this.style) {
        rule = {
          style: this.style
        };
      }

      if (this.exporter) {
        rule.getAttrs = function (dom) {
          var hooks = {};
          exporter.go({
            dom: dom
          }, function (hook) {
            hooks = hook;
          });
          return omit(hooks, ['dom']);
        };
      }

      return rule;
    }
  }, {
    key: "combineRules",
    value: function combineRules() {
      var exporter = this.exporter;
      return this.parseSchema(exporter);
    }
  }]);

  return ParseRule;
}();

var Node = /*#__PURE__*/function () {
  function Node(name) {
    _classCallCheck(this, Node);

    this.name = '';
    this.importer = {};
    this.atom = false;
    this.inline = false;
    this.isolating = false;
    this.draggable = false;
    this.group = '';
    this.content = '';
    this._attrs = {};
    this._parseRules = [];
    this.name = name;
    this.importer = new Middleware();
  }

  _createClass(Node, [{
    key: "fromJSON",
    value: function fromJSON(config) {
      for (var key in config) {
        var value = config[key];
        this[key] = value;
      }
    }
  }, {
    key: "toJSON",
    value: function toJSON() {
      var importer = this.importer;
      return {
        atom: this.atom,
        inline: this.inline,
        group: this.group,
        content: this.content,
        isolating: this.isolating,
        draggable: this.draggable,
        attrs: this._attrs,
        parseDOM: this._parseRules.map(function (rule) {
          return rule.combineRules();
        }),
        toDOM: function toDOM(node) {
          var hooks = {};
          importer.go({
            node: node
          }, function (hook) {
            hooks = hook;
          });
          return hooks.value;
        }
      };
    }
  }, {
    key: "toDOM",
    set: function set(value) {
      this.importer.use(value);
    }
  }, {
    key: "attrs",
    set: function set(value) {
      Object.assign(this._attrs, value);
    }
  }, {
    key: "parseDOM",
    set: function set(parseDom) {
      var _this = this;

      var values = parseDom;

      if (isPlainObject(parseDom)) {
        values = [parseDom];
      }

      values.forEach(function (value) {
        var parseRule = _this._parseRules.find(function (parseRule) {
          if (value.tag) return parseRule.tag === value.tag;
          if (value.style) return parseRule.style === value.style;
          return false;
        });

        if (!parseRule) {
          parseRule = new ParseRule(value);

          _this._parseRules.push(parseRule);
        }

        parseRule.addStack(value.getAttrs);
      });
    }
  }]);

  return Node;
}();

var Mark = /*#__PURE__*/function () {
  function Mark(name) {
    _classCallCheck(this, Mark);

    this.name = '';
    this.importer = {};
    this.inline = false;
    this.group = '';
    this.content = '';
    this.draggable = false;
    this.inclusive = true;
    this.excludes = '';
    this._attrs = {};
    this._parseRules = [];
    this.name = name;
    this.importer = new Middleware();
  }

  _createClass(Mark, [{
    key: "fromJSON",
    value: function fromJSON(config) {
      for (var key in config) {
        var value = config[key];
        this[key] = value;
      }
    }
  }, {
    key: "toJSON",
    value: function toJSON() {
      var importer = this.importer;
      return {
        inline: this.inline,
        group: this.group,
        content: this.content,
        draggable: this.draggable,
        inclusive: this.inclusive,
        excludes: this.excludes,
        attrs: this._attrs,
        parseDOM: this._parseRules.map(function (rule) {
          return rule.combineRules();
        }),
        toDOM: function toDOM(node) {
          var hooks = {};
          importer.go({
            node: node
          }, function (hook) {
            hooks = hook;
          });
          return hooks.value;
        }
      };
    }
  }, {
    key: "toDOM",
    set: function set(value) {
      this.importer.use(value);
    }
  }, {
    key: "attrs",
    set: function set(value) {
      Object.assign(this._attrs, value);
    }
  }, {
    key: "parseDOM",
    set: function set(parseDom) {
      var _this = this;

      var values = parseDom;

      if (isPlainObject(parseDom)) {
        values = [parseDom];
      }

      values.forEach(function (value) {
        var parseRule = _this._parseRules.find(function (parseRule) {
          if (value.tag) return parseRule.tag === value.tag;
          if (value.style) return parseRule.style === value.style;
          return false;
        });

        if (!parseRule) {
          parseRule = new ParseRule(value);

          _this._parseRules.push(parseRule);
        }

        parseRule.addStack(value.getAttrs);
      });
    }
  }]);

  return Mark;
}();

var _dec$1, _class$1, _temp$1;
var Schema = (_dec$1 = injectable(), _dec$1(_class$1 = (_temp$1 = /*#__PURE__*/function () {
  function Schema() {
    _classCallCheck(this, Schema);

    this._nodes = {};
    this._marks = {};
    this.prosemirrorSchema = {
      nodes: {},
      marks: {}
    };
    this.schema = null;
  }

  _createClass(Schema, [{
    key: "addNode",
    value: function addNode(schemaConfig) {
      var name = Object.keys(schemaConfig)[0];
      var config = schemaConfig[name];
      var node = new Node(name);
      var nd = {};

      if (nd = this.has(node)) {
        nd.fromJSON(config);
        return nd;
      } else {
        node.fromJSON(config);
        this.addSchema(node);
        return _defineProperty({}, name, node);
      }
    }
  }, {
    key: "addMark",
    value: function addMark(schemaConfig) {
      var name = Object.keys(schemaConfig)[0];
      var config = schemaConfig[name];
      var mark = new Mark(name);
      var mr = {};

      if (mr = this.has(mark)) {
        mr.fromJSON(config);
        return mr;
      } else {
        mark.fromJSON(config);
        this.addSchema(mark);
        return _defineProperty({}, name, mark);
      }
    }
  }, {
    key: "has",
    value: function has(instance) {
      if (instance instanceof Node) {
        return this._nodes[instance.name] ? this._nodes[instance.name] : false;
      }

      if (instance instanceof Mark) {
        return this._marks[instance.name] ? this._marks[instance.name] : false;
      }
    }
  }, {
    key: "addSchema",
    value: function addSchema(instance) {
      if (instance instanceof Node) {
        return this._nodes[instance.name] ? this._nodes[instance.name] : Object.assign(this._nodes, _defineProperty({}, instance.name, instance));
      }

      if (instance instanceof Mark) {
        return this._marks[instance.name] ? this._marks[instance.name] : Object.assign(this._marks, _defineProperty({}, instance.name, instance));
      }
    }
  }, {
    key: "addProsemirrorSchema",
    value: function addProsemirrorSchema(nodes, type) {
      this.prosemirrorSchema[type] = Object.assign(this.prosemirrorSchema[type], nodes);
    }
  }, {
    key: "getSchema",
    value: function getSchema() {
      var nodes = {};
      var marks = {};

      for (var index in this._nodes) {
        nodes[index] = this._nodes[index].toJSON();
      }

      for (var _index in this._marks) {
        marks[_index] = this._marks[_index].toJSON();
      }

      this.schema = new Schema$1({
        nodes: Object.assign(this.prosemirrorSchema.nodes, nodes),
        marks: Object.assign(this.prosemirrorSchema.marks, marks)
      }); // this.schema = new PmPschema({
      //   nodes: pickBy(
      //     Object.assign(nodes, this.prosemirrorSchema.nodes),
      //     identity,
      //   ),
      //   marks: pickBy(
      //     Object.assign(marks, this.prosemirrorSchema.marks),
      //     identity,
      //   ),
      // });

      return this.schema;
    }
  }]);

  return Schema;
}(), _temp$1)) || _class$1);

// import Mark from './Mark';

var SchemaService = /*#__PURE__*/function (_Service) {
  _inherits(SchemaService, _Service);

  var _super = _createSuper(SchemaService);

  function SchemaService() {
    var _this;

    _classCallCheck(this, SchemaService);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.name = 'SchemaService';
    return _this;
  }

  _createClass(SchemaService, [{
    key: "register",
    value: function register() {
      this.container.bind('Schema').to(Schema).inSingletonScope();
      this.container.bind('CreateNode').toFactory(function (context) {
        return function (schema) {
          var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
            toWaxSchema: false
          };
          var schemaInstance = context.container.get('Schema');

          if (options.toWaxSchema) {
            schemaInstance.addNode(schema);
          } else {
            schemaInstance.addProsemirrorSchema(schema, 'nodes');
          }
        };
      });
      this.container.bind('CreateMark').toFactory(function (context) {
        return function (schema) {
          var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
            toWaxSchema: false
          };
          var schemaInstance = context.container.get('Schema');

          if (options.toWaxSchema) {
            schemaInstance.addMark(schema);
          } else {
            schemaInstance.addProsemirrorSchema(schema, 'marks');
          }
        };
      });
      var createNode = this.container.get('CreateNode');
      var createMark = this.container.get('CreateMark');
      each(this.config, function (schemaElement, type) {
        if (type === 'nodes') {
          createNode(schemaElement);
        } else if (type === 'marks') {
          createMark(schemaElement);
        }
      });
    }
  }]);

  return SchemaService;
}(Service);

var _dec$2, _class$2, _temp$2;

var defaultRules = _toConsumableArray(smartQuotes);

var Rules = (_dec$2 = injectable(), _dec$2(_class$2 = (_temp$2 = /*#__PURE__*/function () {
  function Rules() {
    _classCallCheck(this, Rules);

    this.extendedRules = defaultRules;
  }

  _createClass(Rules, [{
    key: "addRule",
    value: function addRule(rule) {
      var _this$extendedRules;

      (_this$extendedRules = this.extendedRules).push.apply(_this$extendedRules, _toConsumableArray(rule));
    }
  }, {
    key: "createRules",
    value: function createRules() {
      var rulesCreated = inputRules({
        rules: this.extendedRules
      });
      return rulesCreated;
    }
  }]);

  return Rules;
}(), _temp$2)) || _class$2);

var RulesService = /*#__PURE__*/function (_Service) {
  _inherits(RulesService, _Service);

  var _super = _createSuper(RulesService);

  function RulesService() {
    var _this;

    _classCallCheck(this, RulesService);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.name = 'RulesService';
    return _this;
  }

  _createClass(RulesService, [{
    key: "boot",
    value: function boot() {
      var configRules = this.app.config.get('config.RulesService');
      var createRule = this.container.get('CreateRule');
      if (configRules) createRule(configRules);
    }
  }, {
    key: "register",
    value: function register() {
      this.container.bind('Rules').to(Rules).inSingletonScope();
      this.container.bind('CreateRule').toFactory(function (context) {
        return function (rule) {
          var ruleInstance = context.container.get('Rules');
          ruleInstance.addRule(rule);
        };
      });
    }
  }]);

  return RulesService;
}(Service);

var setBlockType = function setBlockType(nodeType) {
  var attrs = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return function (state, dispatch) {
    var tr = state.tr;
    var _state$selection = state.selection,
        from = _state$selection.from,
        to = _state$selection.to;
    state.doc.nodesBetween(from, to, function (node, pos) {
      if (!node.isTextblock || node.hasMarkup(nodeType, attrs)) return;
      var applicable = false;

      if (node.type === nodeType) {
        applicable = true;
      } else {
        var $pos = state.doc.resolve(pos);
        var index = $pos.index();
        applicable = $pos.parent.canReplaceWith(index, index + 1, nodeType);
      }

      if (applicable) {
        tr.setBlockType(from, to, nodeType, _objectSpread2(_objectSpread2({}, node.attrs), attrs));
      }
    });
    if (!tr.steps.length) return false;
    if (dispatch) dispatch(tr.scrollIntoView());
    return true;
  };
};

var markActive = function markActive(type) {
  return function (state) {
    var _state$selection2 = state.selection,
        from = _state$selection2.from,
        $from = _state$selection2.$from,
        to = _state$selection2.to,
        empty = _state$selection2.empty;
    var mark = empty ? type.isInSet(state.storedMarks || $from.marks()) : state.doc.rangeHasMark(from, to, type);
    if (mark) return true;
    return false;
  };
};

var blockActive = function blockActive(nodeType) {
  var attrs = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return function (state) {
    var _state$selection3 = state.selection,
        from = _state$selection3.from,
        to = _state$selection3.to;
    var isActive = false;
    state.doc.nodesBetween(from, to, function (node, pos) {
      if (!node.isTextblock || node.hasMarkup(nodeType, attrs)) return;

      if (node.type === nodeType) {
        isActive = true;
      }
    });
    return isActive;
  };
};

var customTagBlockActive = function customTagBlockActive(type) {
  var attrs = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return function (state) {
    var $from = state.selection.$from;
    if ($from.parent.attrs["class"] === attrs["class"] && $from.parent.type.name === 'customTagBlock') return true;
    return false;
  };
}; // const blockActive = (type, attrs = {}) => state => {
//   const { $from, to, node } = state.selection;
//
//   if (node) {
//     return node.hasMarkup(type, attrs);
//   }
//
//   return to <= $from.end() && $from.parent.hasMarkup(type, attrs);
// };


var canInsert = function canInsert(type) {
  return function (state) {
    var $from = state.selection.$from;

    for (var d = $from.depth; d >= 0; d--) {
      var index = $from.index(d);

      if ($from.node(d).canReplaceWith(index, index, type)) {
        return true;
      }
    }

    return false;
  };
};

var createTable = function createTable(colsRows, state, dispatch) {
  var cells = [];

  while (colsRows.cols--) {
    cells.push(state.config.schema.nodes.table_cell.createAndFill());
  }

  var rows = [];

  while (colsRows.rows--) {
    rows.push(state.config.schema.nodes.table_row.createAndFill(null, cells));
  }

  var tableBody = state.config.schema.nodes.table_body.createAndFill(null, rows);
  dispatch(state.tr.replaceSelectionWith(tableBody));
};

var createLink = function createLink(state, dispatch) {
  var _state$selection4 = state.selection,
      $from = _state$selection4.$from,
      $to = _state$selection4.$to;
  dispatch(state.tr.setMeta('addToHistory', false).addMark($from.pos, $to.pos, state.schema.marks.link.create({
    href: ''
  })));
};

var isOnSameTextBlock = function isOnSameTextBlock(state) {
  var _state$selection5 = state.selection,
      $from = _state$selection5.$from,
      $to = _state$selection5.$to,
      from = _state$selection5.from,
      to = _state$selection5.to;

  if (from !== to && $from.parent === $to.parent && $from.parent.isTextblock) {
    return true;
  }

  return false;
};

var isInTable = function isInTable(state) {
  var $head = state.selection.$head;

  for (var d = $head.depth; d > 0; d -= 1) {
    if ($head.node(d).type.spec.tableRole === 'row') return true;
  }

  return false;
};

var simulateKey = function simulateKey(view, keyCode, key) {
  var event = document.createEvent('Event');
  event.initEvent('keydown', true, true);
  event.keyCode = keyCode;
  event.key = event.code = key;
  return view.someProp('handleKeyDown', function (f) {
    return f(view, event);
  });
};

var isParentOfType = function isParentOfType(state, nodeType) {
  var status = false;

  var predicate = function predicate(node) {
    return node.type === nodeType;
  };

  for (var i = state.selection.$from.depth; i > 0; i -= 1) {
    var node = state.selection.$from.node(i);

    if (predicate(node)) {
      status = true;
    }
  }

  return status;
};

var Commands = {
  isInTable: isInTable,
  setBlockType: setBlockType,
  blockActive: blockActive,
  customTagBlockActive: customTagBlockActive,
  canInsert: canInsert,
  createLink: createLink,
  createTable: createTable,
  markActive: markActive,
  isOnSameTextBlock: isOnSameTextBlock,
  simulateKey: simulateKey,
  isParentOfType: isParentOfType
};

var findMark = function findMark(state, PMmark) {
  var toArr = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var _state$selection = state.selection,
      $from = _state$selection.$from,
      $to = _state$selection.$to,
      doc = state.doc;
  var fromMark = $from.marks().find(function (mark) {
    return mark.type === PMmark;
  });
  var toMark = $to.marks().find(function (mark) {
    return mark.type === PMmark;
  });
  var markFound;
  var marksFound = [];
  doc.nodesBetween($from.pos, $to.pos, function (node, from) {
    if (node.marks) {
      var actualMark = node.marks.find(function (mark) {
        return mark.type === PMmark;
      });

      if (actualMark) {
        markFound = {
          from: from,
          to: from + node.nodeSize,
          attrs: actualMark.attrs,
          contained: !fromMark || !toMark || fromMark === toMark
        };
        marksFound.push(markFound);
      }
    }
  });
  if (toArr) return marksFound;
  return markFound;
};

var findNode = function findNode(state, PMnode, findInParent) {
  var nodeFound;

  if (findInParent) {
    state.doc.nodesBetween(state.selection.from, state.selection.to, function (node, pos) {
      if (node.type.name === 'oen_container' && node.attrs["class"] === 'outline') {
        nodeFound = {
          from: state.selection.from,
          to: state.selection.to,
          node: node
        };
      }
    });
  } else if (state.selection.node && state.selection.node.type.name === PMnode.name) {
    nodeFound = {
      from: state.selection.from,
      to: state.selection.to,
      node: state.selection.node
    };
  }

  return nodeFound;
};

var getCommentsTracksCount = function getCommentsTracksCount(main) {
  var marks = findInlineNodes(main.state.doc);
  var commentsTracksFormat = [];
  var insertionsDeletions = [];
  marks.map(function (node) {
    if (node.node.marks.length > 0) {
      node.node.marks.filter(function (mark) {
        if (mark.type.name === 'comment' || mark.type.name === 'format_change') {
          commentsTracksFormat.push(mark);
        } else if (mark.type.name === 'insertion' || mark.type.name === 'deletion') {
          insertionsDeletions.push(mark);
        }
      });
    }
  });

  var unique = _toConsumableArray(new Set(commentsTracksFormat.map(function (item) {
    return item.attrs.id;
  })));

  var total = unique.length + insertionsDeletions.length;
  return total;
};

var getTrackBlockNodesCount = function getTrackBlockNodesCount(main) {
  var allBlockNodes = findBlockNodes(main.state.doc);
  var trackBlockNodes = [];
  allBlockNodes.map(function (node) {
    if (node.node.attrs.track && node.node.attrs.track.length > 0) {
      trackBlockNodes.push(node);
    }
  });
  return trackBlockNodes.length;
};
/* TODO */

/* this is a hacky workaround for now to find marks
  that are pm will break them. Correct way is to be done
  by the code that is comment out, but sometimes it doesn't return
  the mark altouhgh it works for any other mark. Find out y?
*/


var findFragmentedMark = function findFragmentedMark(state, PMmark) {
  var _state$selection2 = state.selection,
      $from = _state$selection2.$from,
      $to = _state$selection2.$to,
      doc = state.doc; // const type = state.config.schema.marks.comment;
  // const mark = empty
  //   ? type.isInSet(state.storedMarks || $from.marks())
  //   : state.doc.rangeHasMark(from, to, type);
  //
  // if (isObject(mark))
  //   return {
  //     from,
  //     to,
  //     attrs: mark.attrs,
  //   };
  //
  // // return undefined;

  var markFound;
  doc.nodesBetween($from.pos - 1, $to.pos, function (node, from) {
    if (node.marks) {
      var actualMark = node.marks.find(function (mark) {
        return mark.type === PMmark;
      });

      if (actualMark) {
        markFound = {
          from: from,
          to: from + node.nodeSize,
          attrs: actualMark.attrs
        };
      }
    }
  });
  return markFound;
};

var findAllMarksWithSameId = function findAllMarksWithSameId(state, mark) {
  var type = mark.type.name;
  var markType = state.schema.marks[type];
  var allNodes = findChildrenByMark(state.doc, markType, true);
  var allMarksWithSameId = [];
  allNodes.map(function (node) {
    node.node.marks.filter(function (value) {
      if (mark.type.name === type && mark.attrs.id === value.attrs.id) {
        allMarksWithSameId.push(node);
      }
    });
  });
  return allMarksWithSameId;
}; // From https://discuss.prosemirror.net/t/expanding-the-selection-to-the-active-mark/


var findMarkPosition = function findMarkPosition(state, initialPos, markType) {
  var $pos = state.tr.doc.resolve(initialPos);
  var parent = $pos.parent;
  var start = parent.childAfter($pos.parentOffset);
  if (!start.node) return null;
  var actualMark = start.node.marks.find(function (mark) {
    return mark.type.name === markType;
  });
  var startIndex = $pos.index();
  var startPos = $pos.start() + start.offset;

  while (startIndex > 0 && actualMark.isInSet(parent.child(startIndex - 1).marks)) {
    startPos -= parent.child(--startIndex).nodeSize;
  }

  var endIndex = $pos.index() + 1;
  var endPos = $pos.start() + start.offset + start.node.nodeSize;

  while (endIndex < parent.childCount && actualMark.isInSet(parent.child(endIndex).marks)) {
    endPos += parent.child(endIndex++).nodeSize;
  }

  return {
    from: startPos,
    to: endPos,
    attrs: actualMark.attrs
  };
};

var flatten = function flatten(node) {
  var descend = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

  if (!node) {
    throw new Error('Invalid "node" parameter');
  }

  var result = [];
  node.descendants(function (child, pos) {
    result.push({
      node: child,
      pos: pos
    });

    if (!descend) {
      return false;
    }
  });
  return result;
};

var findBlockNodes = function findBlockNodes(node, descend) {
  return findChildren(node, function (child) {
    return child.isBlock;
  }, descend);
};

var findChildrenByType = function findChildrenByType(node, nodeType, descend) {
  return findChildren(node, function (child) {
    return child.type === nodeType;
  }, descend);
};

var findChildren = function findChildren(node, predicate, descend) {
  if (!node) {
    throw new Error('Invalid "node" parameter');
  } else if (!predicate) {
    throw new Error('Invalid "predicate" parameter');
  }

  return flatten(node, descend).filter(function (child) {
    return predicate(child.node);
  });
};

var findInlineNodes = function findInlineNodes(node, descend) {
  return findChildren(node, function (child) {
    return child.isInline;
  }, descend);
};

var findChildrenByMark = function findChildrenByMark(node, markType, descend) {
  return findChildren(node, function (child) {
    return markType.isInSet(child.marks);
  }, descend);
};

var findChildrenByAttr = function findChildrenByAttr(node, predicate, descend) {
  return findChildren(node, function (child) {
    return !!predicate(child.attrs);
  }, descend);
};
/* Find and Replace Matches */

var findMatches = function findMatches(doc, searchValue, matchCase) {
  var allNodes = [];
  doc.descendants(function (node, pos) {
    allNodes.push({
      node: node,
      pos: pos
    });
  });
  eachRight(allNodes, function (node, index) {
    if (node.node.type.groups.includes('notes')) {
      allNodes.splice(index + 1, node.node.childCount);
    }
  });
  var results = [];
  var mergedTextNodes = [];
  var index = 0;
  allNodes.forEach(function (node) {
    if (node.node.isText) {
      if (mergedTextNodes[index]) {
        mergedTextNodes[index] = {
          text: mergedTextNodes[index].text + node.node.text,
          pos: mergedTextNodes[index].pos
        };
      } else {
        mergedTextNodes[index] = {
          text: node.node.text,
          pos: node.pos
        };
      }
    } else {
      index += 1;
    }
  });
  mergedTextNodes.forEach(function (_ref) {
    var text = _ref.text,
        pos = _ref.pos;
    var search = RegExp(escapeRegExp(searchValue), matchCase ? 'gu' : 'gui');
    var m; // eslint-disable-next-line no-cond-assign

    while (m = search.exec(text)) {
      if (m[0] === '') {
        break;
      }

      results.push({
        from: pos + m.index,
        to: pos + m.index + m[0].length
      });
    }
  });
  return results;
};

var escapeRegExp = function escapeRegExp(string) {
  return string.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
};

var findParentOfType = function findParentOfType(state, nodeType) {
  var nodeFound = '';

  var predicate = function predicate(node) {
    return node.type === nodeType;
  };

  for (var i = state.selection.$from.depth; i > 0; i -= 1) {
    var node = state.selection.$from.node(i);

    if (predicate(node)) {
      nodeFound = node;
    }
  }

  return nodeFound;
};

var DocumentHelpers = {
  findMark: findMark,
  findNode: findNode,
  findBlockNodes: findBlockNodes,
  findChildrenByType: findChildrenByType,
  findInlineNodes: findInlineNodes,
  findChildrenByMark: findChildrenByMark,
  findChildrenByAttr: findChildrenByAttr,
  findFragmentedMark: findFragmentedMark,
  findAllMarksWithSameId: findAllMarksWithSameId,
  findMarkPosition: findMarkPosition,
  getCommentsTracksCount: getCommentsTracksCount,
  getTrackBlockNodesCount: getTrackBlockNodesCount,
  findMatches: findMatches,
  findParentOfType: findParentOfType
};

var _dec$3, _class$3, _temp$3;
var backSpace = chainCommands(deleteSelection, joinBackward, selectNodeBackward);

var backSpaceShortCut = function backSpaceShortCut(state, dispatch, view) {
  var _state$selection = state.selection,
      $from = _state$selection.$from,
      $to = _state$selection.$to;
  var nodeBefore = $from.nodeBefore;

  if (nodeBefore && nodeBefore.type.name === 'math_inline') {
    var index = $from.index($from.depth);
    var $beforePos = state.doc.resolve($from.posAtIndex(index - 1));
    dispatch(state.tr.setSelection(new NodeSelection($beforePos)));
    return true;
  }

  state.doc.nodesBetween($from.pos, $to.pos, function (node, from) {
    if (node.type.name === 'fill_the_gap_container' || node.type.name === 'multiple_drop_down_container' || node.type.name === 'numerical_answer_container' || node.type.name === 'essay_container' || node.type.name === 'matching_container') {
      dispatch(state.tr["delete"](from, from + node.nodeSize)); // const index = $from.index($from.depth);
      // const $beforePos = state.doc.resolve($from.posAtIndex(index - 1));
      // dispatch(state.tr.setSelection(new NodeSelection($beforePos)));
    }
  });
  state.doc.nodesBetween($from.pos, $to.pos, function (node, from) {
    if (node.type.name === 'math_display') {
      var $start = state.tr.doc.resolve(state.tr.selection.$anchor.start());
      var $end = state.tr.doc.resolve(state.tr.selection.$anchor.end());
      dispatch(state.tr.setSelection(new TextSelection($start, $end)));
    }
  });
  return backSpace(state, function (tr) {
    return dispatch(tr.setMeta('inputType', 'backwardsDelete'));
  }, view);
};

var pressEnter = function pressEnter(state, dispatch) {
  var _state$selection2 = state.selection,
      $from = _state$selection2.$from,
      to = _state$selection2.to,
      from = _state$selection2.from; // Images

  if (state.doc.resolve(from).parent.type.name === 'figcaption') {
    return true;
  }

  if (state.selection.node && state.selection.node.type.name === 'image') {
    var same = $from.sharedDepth(to);
    var pos = $from.before(same);
    dispatch(state.tr.setSelection(NodeSelection.create(state.doc, pos)));
    return true;
  } // LISTS


  if (splitListItem(state.schema.nodes.list_item)(state)) {
    splitListItem(state.schema.nodes.list_item)(state, dispatch);
    return true;
  } // Title


  if (!Commands.setBlockType(state.config.schema.nodes.title)(state)) {
    var title = DocumentHelpers.findChildrenByType(state.doc, state.config.schema.nodes.title, true);

    if (title.length === 1 && from > title[0].pos + 1 && from < title[0].pos + title[0].node.nodeSize - 1) {
      return true;
    }
  }

  return false;
};

var undoShortCut = function undoShortCut(state, dispatch, view) {
  return undo(state, function (tr) {
    return dispatch(tr.setMeta('inputType', 'Undo'));
  }, view);
};

var redoShortCut = function redoShortCut(state, dispatch, view) {
  return redo(state, function (tr) {
    return dispatch(tr.setMeta('inputType', 'Redo'));
  }, view);
};

var insertBreak = function insertBreak(state, dispatch) {
  var br = state.schema.nodes.hard_break.create();
  dispatch(state.tr.replaceSelectionWith(br).scrollIntoView());
  return true;
}; // const insertRule = (state, dispatch) => {
//   const hr = this.schema.nodes.horizontal_rule.create();
//   dispatch(state.tr.replaceSelectionWith(hr).scrollIntoView());
//   return true;
// }


var getKeys = {
  'Mod-z': undoShortCut,
  'Shift-Mod-z': redoShortCut,
  Backspace: backSpaceShortCut,
  'Mod-y': redoShortCut,
  Escape: selectParentNode,
  'Mod-Enter': chainCommands(exitCode, insertBreak),
  'Shift-Enter': chainCommands(exitCode, insertBreak),
  'Ctrl-Enter': chainCommands(exitCode, insertBreak),
  // 'Mod-_': this.insertRule,
  Enter: pressEnter
};
var ShortCuts = (_dec$3 = injectable(), _dec$3(_class$3 = (_temp$3 = /*#__PURE__*/function () {
  function ShortCuts() {
    _classCallCheck(this, ShortCuts);

    this.keys = getKeys;
  }

  _createClass(ShortCuts, [{
    key: "createShortCuts",
    value: function createShortCuts() {
      return keymap(this.createKeyBindings());
    }
  }, {
    key: "addShortCut",
    value: function addShortCut(shortcut) {
      Object.assign(this.keys, shortcut);
    }
  }, {
    key: "createKeyBindings",
    value: function createKeyBindings() {
      var _this = this;

      Object.keys(baseKeymap).forEach(function (key) {
        if (_this.keys[key]) {
          _this.keys[key] = chainCommands(_this.keys[key], baseKeymap[key]);
        } else {
          _this.keys[key] = baseKeymap[key];
        }
      });
      return this.keys;
    }
  }]);

  return ShortCuts;
}(), _temp$3)) || _class$3);

var ShortCutsService = /*#__PURE__*/function (_Service) {
  _inherits(ShortCutsService, _Service);

  var _super = _createSuper(ShortCutsService);

  function ShortCutsService() {
    var _this;

    _classCallCheck(this, ShortCutsService);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.name = 'ShortCutsService';
    return _this;
  }

  _createClass(ShortCutsService, [{
    key: "register",
    value: function register() {
      this.container.bind('ShortCuts').to(ShortCuts).inSingletonScope();
      this.container.bind('CreateShortCut').toFactory(function (context) {
        return function (shortCut) {
          var shortCutsInstance = context.container.get('ShortCuts');
          shortCutsInstance.addShortCut(shortCut);
        };
      });
    }
  }]);

  return ShortCutsService;
}(Service);

/* eslint-disable react/prop-types */

var DefaultLayout = function DefaultLayout(_ref) {
  var editor = _ref.editor;
  return /*#__PURE__*/React.createElement(React.Fragment, null, editor);
};

/* eslint-disable react/jsx-props-no-spreading */
var LayoutFactory = (function (Component) {
  return function (props) {
    return Component;
  };
});

var _dec$4, _class$4, _temp$4;
var Layout = (_dec$4 = injectable(), _dec$4(_class$4 = (_temp$4 = /*#__PURE__*/function () {
  function Layout() {
    _classCallCheck(this, Layout);

    this.components = [];
    this.layoutComponent = LayoutFactory(DefaultLayout);
  }

  _createClass(Layout, [{
    key: "addComponent",
    value: function addComponent(renderArea, component, componentProps) {
      if (!this.components[renderArea]) {
        this.createArea(renderArea);
      }

      var size = this.components[renderArea].size;
      this.components[renderArea].set(size + 1, {
        component: component,
        componentProps: componentProps
      });
      return this;
    }
  }, {
    key: "render",
    value: function render(renderArea) {
      if (!this.components[renderArea]) return null;
      return this.getArray(this.components[renderArea]);
    }
  }, {
    key: "createArea",
    value: function createArea(area) {
      this.components[area] = new Map();
    } // eslint-disable-next-line class-methods-use-this

  }, {
    key: "getArray",
    value: function getArray(iterator) {
      var components = [];
      iterator.forEach(function (component) {
        return components.push(component);
      });
      return components;
    }
  }, {
    key: "setLayout",
    value: function setLayout(component) {
      this.layoutComponent = LayoutFactory(component);
    }
  }]);

  return Layout;
}(), _temp$4)) || _class$4);

var LayoutService = /*#__PURE__*/function (_Service) {
  _inherits(LayoutService, _Service);

  var _super = _createSuper(LayoutService);

  function LayoutService() {
    var _this;

    _classCallCheck(this, LayoutService);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.name = 'LayoutService';
    return _this;
  }

  _createClass(LayoutService, [{
    key: "register",
    value: function register() {
      this.container.bind('Layout').to(Layout).inSingletonScope();
    }
  }]);

  return LayoutService;
}(Service);

var PortalComponent = (function () {
  var _useContext = useContext(PortalContext),
      portals = _useContext.portals;

  return /*#__PURE__*/React.createElement(React.Fragment, null, portals.length > 0 && portals.map(function (_ref) {
    var element = _ref.element,
        Component = _ref.component,
        id = _ref.id,
        decorations = _ref.decorations,
        node = _ref.node,
        view = _ref.view,
        getPos = _ref.getPos;
    return ReactDOM.createPortal( /*#__PURE__*/React.createElement(Component, {
      decorations: decorations,
      getPos: getPos,
      key: id,
      node: node,
      view: view
    }), element, id);
  }));
});

var AbstractNodeView = /*#__PURE__*/function () {
  function AbstractNodeView(node, view, getPos, decorations, createPortal, Component, context) {
    _classCallCheck(this, AbstractNodeView);

    this.dom = node.type.spec.inline ? document.createElement('span') : document.createElement('div');
    this.dom.id = v4();
    this.dom.classList.add('portal');
    this.node = node;
    this.outerView = view;
    this.getPos = getPos;
    this.context = context;
    createPortal(this.dom, Component, node, view, getPos, decorations, context);
  }

  _createClass(AbstractNodeView, [{
    key: "update",
    value: function update(node) {
      if (node.type !== this.node.type) return false;
      return true;
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.dom = undefined;
      this.contentDOM = undefined;
    }
  }, {
    key: "stopEvent",
    value: function stopEvent(event) {
      var innerView = this.context.pmViews[this.node.attrs.id];
      return innerView && innerView.dom.contains(event.target);
    } // eslint-disable-next-line class-methods-use-this

  }, {
    key: "ignoreMutation",
    value: function ignoreMutation() {
      return true;
    }
  }, {
    key: "selectNode",
    value: function selectNode() {
      this.dom.classList.add('ProseMirror-selectednode');
    }
  }, {
    key: "deselectNode",
    value: function deselectNode() {
      this.dom.classList.remove('ProseMirror-selectednode');
    }
  }]);

  return AbstractNodeView;
}();

var portalPlugin = new PluginKey('portalPlugin');

var CreateNodeView = function CreateNodeView(createPortal, Component, NodeView, context) {
  return function (theNode, view, getPos, decorations) {
    return new NodeView(theNode, view, getPos, decorations, createPortal, Component, context);
  };
};

var PortalPlugin = (function (props) {
  var nodeViews = {};
  props.portals.forEach(function (portal) {
    var name = portal.nodeView ? portal.nodeView.name() : portal.name;
    nodeViews[name] = CreateNodeView(props.createPortal, portal.component, portal.nodeView || AbstractNodeView, portal.context.context);
  });
  return new Plugin({
    key: portalPlugin,
    props: {
      nodeViews: nodeViews
    }
  });
});

var _dec$5, _class$5, _temp$5;
var Portals = (_dec$5 = injectable(), _dec$5(_class$5 = (_temp$5 = /*#__PURE__*/function () {
  function Portals() {
    _classCallCheck(this, Portals);

    this.portals = [];
  }

  _createClass(Portals, [{
    key: "addPortal",
    value: function addPortal(portal) {
      this.portals.push(portal);
    }
  }, {
    key: "getPortals",
    value: function getPortals() {
      return this.portals;
    }
  }]);

  return Portals;
}(), _temp$5)) || _class$5);

var PortalService = /*#__PURE__*/function (_Service) {
  _inherits(PortalService, _Service);

  var _super = _createSuper(PortalService);

  function PortalService() {
    _classCallCheck(this, PortalService);

    return _super.apply(this, arguments);
  }

  _createClass(PortalService, [{
    key: "boot",
    value: function boot() {
      var portals = this.container.get('Portals');
      this.app.PmPlugins.add('portalPlugin', PortalPlugin({
        createPortal: this.app.context.createPortal,
        portals: portals.getPortals()
      }));
    }
  }, {
    key: "register",
    value: function register() {
      var layout = this.container.get('Layout');
      layout.addComponent('waxPortals', PortalComponent);
      this.container.bind('Portals').to(Portals).inSingletonScope();
      this.container.bind('AddPortal').toFactory(function (context) {
        return function (portal) {
          var portalInstance = context.container.get('Portals');
          portalInstance.addPortal(portal);
        };
      });
    }
  }]);

  return PortalService;
}(Service);

function _taggedTemplateLiteral(strings, raw) {
  if (!raw) {
    raw = strings.slice(0);
  }

  return Object.freeze(Object.defineProperties(strings, {
    raw: {
      value: Object.freeze(raw)
    }
  }));
}

function _templateObject() {
  var data = _taggedTemplateLiteral(["\n  fill: ", ";\n  height: 24px;\n  vertical-align: top;\n  width: 24px;\n"]);

  _templateObject = function _templateObject() {
    return data;
  };

  return data;
}
var Svg = styled.svg.attrs(function () {
  return {
    version: '1.1',
    xmlns: 'http://www.w3.org/2000/svg',
    xmlnsXlink: 'http://www.w3.org/1999/xlink'
  };
})(_templateObject(), th('colorPrimary'));

var Translation = function Translation(_ref) {
  var label = _ref.label,
      defaultTrans = _ref.defaultTrans;

  var _useTranslation = useTranslation(),
      t = _useTranslation.t,
      i18n = _useTranslation.i18n;

  return /*#__PURE__*/React.createElement(React.Fragment, null, !isEmpty(i18n) && i18n.exists(label) ? t(label) : defaultTrans);
};

var icons = {
  commentBubble: function commentBubble(_ref2) {
    var className = _ref2.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M20 2H4c-1.1 0-1.99.9-1.99 2L2 22l4-4h14c1.1 0 2-.9 2-2V4c0-1.1-.9-2-2-2zM6 9h12v2H6V9zm8 5H6v-2h8v2zm4-6H6V6h12v2z"
    }));
  },
  blockquote: function blockquote(_ref3) {
    var className = _ref3.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M.78,8.89c0-3.07,1.53-4.3,4.3-4.34L5.38,6C3.78,6.17,3,7,3.1,8.31H4.54V12H.78Zm5.9,0c0-3.07,1.53-4.3,4.3-4.34L11.28,6C9.68,6.17,8.89,7,9,8.31h1.44V12H6.68Z"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M16.94,15.11c0,3.07-1.53,4.3-4.3,4.34L12.35,18c1.6-.16,2.39-1,2.28-2.3H13.18V12h3.76Zm5.9,0c0,3.07-1.53,4.3-4.3,4.34L18.24,18c1.6-.16,2.39-1,2.28-2.3H19.08V12h3.76Z"
    }));
  },
  bold: function bold(_ref4) {
    var className = _ref4.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M15.6 10.79c.97-.67 1.65-1.77 1.65-2.79 0-2.26-1.75-4-4-4H7v14h7.04c2.09 0 3.71-1.7 3.71-3.79 0-1.52-.86-2.82-2.15-3.42zM10 6.5h3c.83 0 1.5.67 1.5 1.5s-.67 1.5-1.5 1.5h-3v-3zm3.5 9H10v-3h3.5c.83 0 1.5.67 1.5 1.5s-.67 1.5-1.5 1.5z"
    }));
  },
  italic: function italic(_ref5) {
    var className = _ref5.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M10 4v3h2.21l-3.42 8H6v3h8v-3h-2.21l3.42-8H18V4z"
    }));
  },
  undo: function undo(_ref6) {
    var className = _ref6.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M12.5 8c-2.65 0-5.05.99-6.9 2.6L2 7v9h9l-3.62-3.62c1.39-1.16 3.16-1.88 5.12-1.88 3.54 0 6.55 2.31 7.6 5.5l2.37-.78C21.08 11.03 17.15 8 12.5 8z"
    }));
  },
  redo: function redo(_ref7) {
    var className = _ref7.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M18.4 10.6C16.55 8.99 14.15 8 11.5 8c-4.65 0-8.58 3.03-9.96 7.22L3.9 16c1.05-3.19 4.05-5.5 7.6-5.5 1.95 0 3.73.72 5.12 1.88L13 16h9V7l-3.6 3.6z"
    }));
  },
  save: function save(_ref8) {
    var className = _ref8.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M17 3H5c-1.11 0-2 .9-2 2v14c0 1.1.89 2 2 2h14c1.1 0 2-.9 2-2V7l-4-4zm-5 16c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3zm3-10H5V5h10v4z"
    }));
  },
  done: function done(_ref9) {
    var className = _ref9.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M9 16.2L4.8 12l-1.4 1.4L9 19 21 7l-1.4-1.4L9 16.2z"
    }));
  },
  code: function code(_ref10) {
    var className = _ref10.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0V0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M9.4 16.6L4.8 12l4.6-4.6L8 6l-6 6 6 6 1.4-1.4zm5.2 0l4.6-4.6-4.6-4.6L16 6l6 6-6 6-1.4-1.4z"
    }));
  },
  link: function link(_ref11) {
    var className = _ref11.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M3.9 12c0-1.71 1.39-3.1 3.1-3.1h4V7H7c-2.76 0-5 2.24-5 5s2.24 5 5 5h4v-1.9H7c-1.71 0-3.1-1.39-3.1-3.1zM8 13h8v-2H8v2zm9-6h-4v1.9h4c1.71 0 3.1 1.39 3.1 3.1s-1.39 3.1-3.1 3.1h-4V17h4c2.76 0 5-2.24 5-5s-2.24-5-5-5z"
    }));
  },
  strikethrough: function strikethrough(_ref12) {
    var className = _ref12.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      "enable-background": "new 0 0 24 24",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("rect", {
      fill: "none",
      height: "24",
      width: "24"
    })), /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("path", {
      d: "M6.85,7.08C6.85,4.37,9.45,3,12.24,3c1.64,0,3,0.49,3.9,1.28c0.77,0.65,1.46,1.73,1.46,3.24h-3.01 c0-0.31-0.05-0.59-0.15-0.85c-0.29-0.86-1.2-1.28-2.25-1.28c-1.86,0-2.34,1.02-2.34,1.7c0,0.48,0.25,0.88,0.74,1.21 C10.97,8.55,11.36,8.78,12,9H7.39C7.18,8.66,6.85,8.11,6.85,7.08z M21,12v-2H3v2h9.62c1.15,0.45,1.96,0.75,1.96,1.97 c0,1-0.81,1.67-2.28,1.67c-1.54,0-2.93-0.54-2.93-2.51H6.4c0,0.55,0.08,1.13,0.24,1.58c0.81,2.29,3.29,3.3,5.67,3.3 c2.27,0,5.3-0.89,5.3-4.05c0-0.3-0.01-1.16-0.48-1.94H21V12z"
    })))));
  },
  underline: function underline(_ref13) {
    var className = _ref13.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M12 17c3.31 0 6-2.69 6-6V3h-2.5v8c0 1.93-1.57 3.5-3.5 3.5S8.5 12.93 8.5 11V3H6v8c0 3.31 2.69 6 6 6zm-7 2v2h14v-2H5z"
    }));
  },
  more: function more(_ref14) {
    var className = _ref14.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "more",
      label: "Wax.Various.more"
    })), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M6 10c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12 0c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm-6 0c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"
    }));
  },
  subscript: function subscript(_ref15) {
    var className = _ref15.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      "enable-background": "new 0 0 24 24",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("rect", {
      fill: "none",
      height: "24",
      width: "24"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M22,18h-2v1h3v1h-4v-2c0-0.55,0.45-1,1-1h2v-1h-3v-1h3c0.55,0,1,0.45,1,1v1C23,17.55,22.55,18,22,18z M5.88,18h2.66 l3.4-5.42h0.12l3.4,5.42h2.66l-4.65-7.27L17.81,4h-2.68l-3.07,4.99h-0.12L8.85,4H6.19l4.32,6.73L5.88,18z"
    })));
  },
  superscript: function superscript(_ref16) {
    var className = _ref16.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      "enable-background": "new 0 0 24 24",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("rect", {
      fill: "none",
      height: "24",
      width: "24",
      x: "0",
      y: "0"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M22,7h-2v1h3v1h-4V7c0-0.55,0.45-1,1-1h2V5h-3V4h3c0.55,0,1,0.45,1,1v1C23,6.55,22.55,7,22,7z M5.88,20h2.66l3.4-5.42h0.12 l3.4,5.42h2.66l-4.65-7.27L17.81,6h-2.68l-3.07,4.99h-0.12L8.85,6H6.19l4.32,6.73L5.88,20z"
    })));
  },
  smallCaps: function smallCaps(_ref17) {
    var className = _ref17.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      "enable-background": "new 0 0 24 24",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("rect", {
      fill: "none",
      height: "24",
      width: "24"
    })), /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("path", {
      d: "M2.5,4v3h5v12h3V7h5V4H2.5z M21.5,9h-9v3h3v7h3v-7h3V9z"
    })))));
  },
  note: function note(_ref18) {
    var className = _ref18.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0V0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M22 10l-6-6H4c-1.1 0-2 .9-2 2v12.01c0 1.1.9 1.99 2 1.99l16-.01c1.1 0 2-.89 2-1.99v-8zm-7-4.5l5.5 5.5H15V5.5z"
    }));
  },
  numberedList: function numberedList(_ref19) {
    var className = _ref19.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M2 17h2v.5H3v1h1v.5H2v1h3v-4H2v1zm1-9h1V4H2v1h1v3zm-1 3h1.8L2 13.1v.9h3v-1H3.2L5 10.9V10H2v1zm5-6v2h14V5H7zm0 14h14v-2H7v2zm0-6h14v-2H7v2z"
    }));
  },
  bulletList: function bulletList(_ref20) {
    var className = _ref20.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0V0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M4 10.5c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm0-6c-.83 0-1.5.67-1.5 1.5S3.17 7.5 4 7.5 5.5 6.83 5.5 6 4.83 4.5 4 4.5zm0 12c-.83 0-1.5.68-1.5 1.5s.68 1.5 1.5 1.5 1.5-.68 1.5-1.5-.67-1.5-1.5-1.5zM7 19h14v-2H7v2zm0-6h14v-2H7v2zm0-8v2h14V5H7z"
    }));
  },
  indentDecrease: function indentDecrease(_ref21) {
    var className = _ref21.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M11 17h10v-2H11v2zm-8-5l4 4V8l-4 4zm0 9h18v-2H3v2zM3 3v2h18V3H3zm8 6h10V7H11v2zm0 4h10v-2H11v2z"
    }));
  },
  image: function image(_ref22) {
    var className = _ref22.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M21 19V5c0-1.1-.9-2-2-2H5c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2zM8.5 13.5l2.5 3.01L14.5 12l4.5 6H5l3.5-4.5z"
    }));
  },
  table: function table(_ref23) {
    var className = _ref23.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M3 3v18h18V3H3zm8 16H5v-6h6v6zm0-8H5V5h6v6zm8 8h-6v-6h6v6zm0-8h-6V5h6v6z"
    }));
  },

  /* -- table alternative icon -- */
  // table: ({ className }) => (
  //   <Svg className={className} viewBox="0 0 24 24">
  //     <path d="M0 0h24v24H0V0z" fill="none" />
  //     <path d="M10 10.02h5V21h-5zM17 21h3c1.1 0 2-.9 2-2v-9h-5v11zm3-18H5c-1.1 0-2 .9-2 2v3h19V5c0-1.1-.9-2-2-2zM3 19c0 1.1.9 2 2 2h3V10H3v9z" />
  //   </Svg>
  // ),
  arrowUp: function arrowUp(_ref24) {
    var className = _ref24.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M7.41 15.41L12 10.83l4.59 4.58L18 14l-6-6-6 6z"
    }));
  },
  arrowDown: function arrowDown(_ref25) {
    var className = _ref25.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0V0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M7.41 8.59L12 13.17l4.59-4.58L18 10l-6 6-6-6 1.41-1.41z"
    }));
  },
  codeBlock: function codeBlock(_ref26) {
    var className = _ref26.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      "enable-background": "new 0 0 24 24",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("rect", {
      fill: "none",
      height: "24",
      width: "24"
    }), /*#__PURE__*/React.createElement("circle", {
      cx: "12",
      cy: "3.5",
      fill: "none",
      r: ".75"
    }), /*#__PURE__*/React.createElement("circle", {
      cx: "12",
      cy: "3.5",
      fill: "none",
      r: ".75"
    }), /*#__PURE__*/React.createElement("circle", {
      cx: "12",
      cy: "3.5",
      fill: "none",
      r: ".75"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M19,3h-4.18C14.4,1.84,13.3,1,12,1S9.6,1.84,9.18,3H5C4.86,3,4.73,3.01,4.6,3.04C4.21,3.12,3.86,3.32,3.59,3.59 c-0.18,0.18-0.33,0.4-0.43,0.64C3.06,4.46,3,4.72,3,5v14c0,0.27,0.06,0.54,0.16,0.78c0.1,0.24,0.25,0.45,0.43,0.64 c0.27,0.27,0.62,0.47,1.01,0.55C4.73,20.99,4.86,21,5,21h14c1.1,0,2-0.9,2-2V5C21,3.9,20.1,3,19,3z M11,14.17l-1.41,1.42L6,12 l3.59-3.59L11,9.83L8.83,12L11,14.17z M12,4.25c-0.41,0-0.75-0.34-0.75-0.75S11.59,2.75,12,2.75s0.75,0.34,0.75,0.75 S12.41,4.25,12,4.25z M14.41,15.59L13,14.17L15.17,12L13,9.83l1.41-1.42L18,12L14.41,15.59z"
    })));
  },
  highlight: function highlight(_ref27) {
    var className = _ref27.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M14.837,2.538L7.587,9.617L6.826,11.846L5.707,12.94L8.752,15.912L9.869,14.818L12.153,14.075L19.402,6.995L14.837,2.538ZM21.685,6.252C22.105,6.662 22.105,7.328 21.685,7.738L13.315,15.912L11.033,16.655L9.511,18.141C9.091,18.551 8.41,18.551 7.989,18.141L3.424,13.682C3.004,13.272 3.004,12.607 3.424,12.197L4.945,10.711L5.706,8.482L14.076,0.308C14.497,-0.103 15.178,-0.103 15.598,0.308L21.685,6.252ZM14.837,5.509L16.359,6.995L10.46,12.769L8.939,11.283L14.837,5.509ZM6.468,18L4.566,20L0,18.514L3.424,15.027",
      id: "highlight"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M7.447,18.31L7.64,18.499C8.254,19.097 9.248,19.097 9.86,18.499L10.053,18.31L22.506,18.31L22.506,21.417L1.876,21.417L1.876,19.65L4.411,20.475C4.594,20.535 4.796,20.484 4.928,20.345L6.83,18.345L6.794,18.31L7.447,18.31Z",
      id: "trait"
    }));
  },
  title: function title(_ref28) {
    var className = _ref28.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0V0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M5 4v3h5.5v12h3V7H19V4z"
    }));
  },
  findAndReplace: function findAndReplace(_ref29) {
    var className = _ref29.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M11 6c1.38 0 2.63.56 3.54 1.46L12 10h6V4l-2.05 2.05C14.68 4.78 12.93 4 11 4c-3.53 0-6.43 2.61-6.92 6H6.1c.46-2.28 2.48-4 4.9-4zm5.64 9.14c.66-.9 1.12-1.97 1.28-3.14H15.9c-.46 2.28-2.48 4-4.9 4-1.38 0-2.63-.56-3.54-1.46L10 12H4v6l2.05-2.05C7.32 17.22 9.07 18 11 18c1.55 0 2.98-.51 4.14-1.36L20 21.49 21.49 20l-4.85-4.86z"
    }));
  },
  navigatePrevious: function navigatePrevious(_ref30) {
    var className = _ref30.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Previous",
      label: "Wax.FindAndReplace.Previous"
    })), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M15.41 7.41L14 6l-6 6 6 6 1.41-1.41L10.83 12z"
    }));
  },
  navigateNext: function navigateNext(_ref31) {
    var className = _ref31.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Next",
      label: "Wax.FindAndReplace.Next"
    })), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0V0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M10.02 6L8.61 7.41 13.19 12l-4.58 4.59L10.02 18l6-6-6-6z"
    }));
  },
  close: function close(_ref32) {
    var className = _ref32.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Close",
      label: "Wax.Various.Close"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0V0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12 19 6.41z"
    }));
  },
  reject: function reject(_ref33) {
    var className = _ref33.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Reject",
      label: "Wax.TrackChanges.Reject"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0V0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12 19 6.41z"
    }));
  },
  removeTag: function removeTag(_ref34) {
    var className = _ref34.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "remove tag",
      label: "Wax.Various.remove tag"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0V0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12 19 6.41z"
    }));
  },
  fullScreen: function fullScreen(_ref35) {
    var className = _ref35.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Full screen",
      label: "Wax.Various.Full screen"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M7 14H5v5h5v-2H7v-3zm-2-4h2V7h3V5H5v5zm12 7h-3v2h5v-5h-2v3zM14 5v2h3v3h2V5h-5z"
    }));
  },
  fullScreenExit: function fullScreenExit(_ref36) {
    var className = _ref36.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Exit full screen",
      label: "Wax.Various.Exit full screen"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M5 16h3v3h2v-5H5v2zm3-8H5v2h5V5H8v3zm6 11h2v-3h3v-2h-5v5zm2-11V5h-2v5h5V8h-3z"
    }));
  },
  editing: function editing(_ref37) {
    var className = _ref37.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Editing",
      label: "Wax.TrackChanges.Editing"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M3 17.25V21h3.75L17.81 9.94l-3.75-3.75L3 17.25zM20.71 7.04c.39-.39.39-1.02 0-1.41l-2.34-2.34c-.39-.39-1.02-.39-1.41 0l-1.83 1.83 3.75 3.75 1.83-1.83z"
    }));
  },
  suggesting: function suggesting(_ref38) {
    var className = _ref38.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Suggesting",
      label: "Wax.TrackChanges.Suggesting"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M22,24H2v-4h20V24z M13.06,5.19l3.75,3.75L7.75,18H4v-3.75L13.06,5.19z M17.88,7.87l-3.75-3.75 l1.83-1.83c0.39-0.39,1.02-0.39,1.41,0l2.34,2.34c0.39,0.39,0.39,1.02,0,1.41L17.88,7.87z"
    }));
  },
  viewing: function viewing(_ref39) {
    var className = _ref39.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Viewing",
      label: "Wax.Various.Viewing"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M12 4.5C7 4.5 2.73 7.61 1 12c1.73 4.39 6 7.5 11 7.5s9.27-3.11 11-7.5c-1.73-4.39-6-7.5-11-7.5zM12 17c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5zm0-8c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3z"
    }));
  },
  toggleOn: function toggleOn(_ref40) {
    var className = _ref40.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Viewing",
      label: "Wax.Various.Viewing"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M17 7H7c-2.76 0-5 2.24-5 5s2.24 5 5 5h10c2.76 0 5-2.24 5-5s-2.24-5-5-5zm0 8c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3z"
    }));
  },
  specialCharacters: function specialCharacters(_ref41) {
    var className = _ref41.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("rect", {
      fill: "none",
      height: "24",
      width: "24",
      x: "0"
    })), /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("path", {
      d: "M16.8279 5.64748C16.1334 5.21583 15.306 5 14.3455 5C13.5476 5 12.8383 5.16547 12.2177 5.4964C11.5971 5.82734 11.0947 6.28777 10.7106 6.8777C10.4003 6.23022 9.9422 5.76259 9.33638 5.47482C8.74533 5.17266 7.99174 5.02158 7.07562 5.02158C6.35159 5.02158 5.62017 5.11511 4.88136 5.30216C4.14255 5.48921 3.51456 5.72662 2.99739 6.01439L3.59583 7.43885C4.08344 7.19424 4.608 6.98561 5.16949 6.81295C5.74576 6.64029 6.32942 6.55396 6.92047 6.55396C7.67406 6.55396 8.27249 6.74101 8.71578 7.11511C9.17384 7.48921 9.40287 8.14389 9.40287 9.07914V9.81295L7.40808 9.8777C6.19644 9.90647 5.18427 10.0719 4.37158 10.3741C3.57366 10.6619 2.97523 11.0719 2.57627 11.6043C2.19209 12.1367 2 12.7914 2 13.5683C2 14.3453 2.16254 14.9928 2.48761 15.5108C2.82747 16.0144 3.27814 16.3885 3.83964 16.6331C4.41591 16.8777 5.05867 17 5.76793 17C6.53629 17 7.18644 16.8993 7.71838 16.6978C8.2651 16.4964 8.73794 16.2158 9.1369 15.8561C9.55063 15.482 9.92742 15.0504 10.2673 14.5612C10.5776 15.0935 10.947 15.5468 11.3755 15.9209C11.8188 16.2806 12.3212 16.554 12.8827 16.741C13.4589 16.9137 14.0869 17 14.7666 17C14.8924 17 15.0157 16.9981 15.1364 16.9944V15.3775C15.037 15.38 14.9359 15.3813 14.8331 15.3813C13.7397 15.3813 12.8974 15.0576 12.3064 14.4101C11.7301 13.7482 11.4272 12.7266 11.3977 11.3453H19V10.1799C19 9.14388 18.8079 8.23741 18.4237 7.46043C18.0543 6.68345 17.5224 6.07914 16.8279 5.64748ZM4.81486 11.8849C5.34681 11.482 6.30726 11.2518 7.69622 11.1942L9.35854 11.1295V12.1655C9.35854 12.9137 9.21078 13.5324 8.91525 14.0216C8.63451 14.5108 8.24294 14.8777 7.74055 15.1223C7.25293 15.3525 6.69883 15.4676 6.07823 15.4676C5.45763 15.4676 4.95524 15.3165 4.57106 15.0144C4.20165 14.7122 4.01695 14.2374 4.01695 13.5899C4.01695 12.8561 4.28292 12.2878 4.81486 11.8849ZM12.2842 7.41727C12.7866 6.84173 13.4663 6.55396 14.3233 6.55396C14.9292 6.55396 15.4242 6.69065 15.8083 6.96403C16.1925 7.23741 16.4733 7.6259 16.6506 8.1295C16.8427 8.61871 16.9387 9.19424 16.9387 9.85611H11.442C11.5159 8.79137 11.7966 7.97842 12.2842 7.41727Z"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M20.75 15.25V13H19.25V15.25H17V16.75H19.25V19H20.75V16.75H23V15.25H20.75Z"
    })))));
  },
  chapterList: function chapterList(_ref42) {
    var className = _ref42.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Chapters",
      label: "Wax.Various.Chapters"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M14 2H6c-1.1 0-1.99.9-1.99 2L4 20c0 1.1.89 2 1.99 2H18c1.1 0 2-.9 2-2V8l-6-6zm2 16H8v-2h8v2zm0-4H8v-2h8v2zm-3-5V3.5L18.5 9H13z"
    }));
  },
  showTrack: function showTrack(_ref43) {
    var className = _ref43.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Show Changes",
      label: "Wax.TrackChanges.Show Changes"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M 18 2.016 L 18 0 L 0 0 L 0 2.016 Z M 18 6.984 L 18 5.016 L 0 5.016 L 0 6.984 Z M 7.5 12 L 7.5 9.984 L 0 9.984 L 0 12 Z M 16.871 11.691 L 17.633 10.93 C 17.711 10.852 17.75 10.754 17.75 10.637 C 17.75 10.52 17.711 10.422 17.633 10.344 L 17.633 10.344 L 16.656 9.367 C 16.578 9.289 16.48 9.25 16.363 9.25 C 16.246 9.25 16.148 9.289 16.07 9.367 L 16.07 9.367 L 15.309 10.129 Z M 11.813 16.75 L 16.422 12.141 L 14.859 10.578 L 10.25 15.188 L 10.25 16.75 Z"
    }));
  },
  acceptTrack: function acceptTrack(_ref44) {
    var className = _ref44.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Accept",
      label: "Wax.TrackChanges.Accept"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M 18 2.016 L 18 0 L 0 0 L 0 2.016 Z M 18 6.984 L 18 5.016 L 0 5.016 L 0 6.984 Z M 7.5 12 L 7.5 9.984 L 0 9.984 L 0 12 Z M 12.563 17.078 L 18.188 11.406 L 16.781 10 L 12.563 14.219 L 10.406 12.109 L 9 13.516 Z"
    }));
  },
  rejectTrack: function rejectTrack(_ref45) {
    var className = _ref45.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Reject",
      label: "Wax.TrackChanges.Reject"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M 18 2.016 L 18 0 L 0 0 L 0 2.016 Z M 18 6.984 L 18 5.016 L 0 5.016 L 0 6.984 Z M 7.5 12 L 7.5 9.984 L 0 9.984 L 0 12 Z M 16.609 17.516 L 18.016 16.109 L 15.438 13.484 L 18.016 10.906 L 16.609 9.5 L 14.031 12.078 L 11.406 9.5 L 10 10.906 L 12.625 13.484 L 10 16.109 L 11.406 17.516 L 14.031 14.891 Z"
    }));
  },
  acceptRejectTrack: function acceptRejectTrack(_ref46) {
    var className = _ref46.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Accept",
      label: "Wax.TrackChanges.Accept"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M 14.531 7.078 L 20.156 1.406 L 18.75 0 L 14.531 4.219 L 12.375 2.109 L 10.969 3.516 Z M 9 5.063 L 9 3.047 L 0 3.047 L 0 5.063 Z M 17.578 16.078 L 18.984 14.672 L 16.406 12.047 L 18.984 9.469 L 17.578 8.063 L 15 10.641 L 12.375 8.063 L 10.969 9.469 L 13.594 12.047 L 10.969 14.672 L 12.375 16.078 L 15 13.453 Z M 9 13.078 L 9 11.063 L 0 11.063 L 0 13.078 Z"
    }));
  },
  checkTrack: function checkTrack(_ref47) {
    var className = _ref47.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Accept",
      label: "Wax.TrackChanges.Accept"
    }), ' '), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M9 16.17L4.83 12l-1.42 1.41L9 19 21 7l-1.41-1.41z"
    }));
  },
  transformCase: function transformCase(_ref48) {
    var className = _ref48.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      enableBackground: "new 0 0 16 16",
      fill: "none",
      viewBox: "0 0 28 28"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M11.44,15.353c0-2.99-1.352-5.486-5.59-5.486c-2.314,0-4.056,0.624-4.94,1.118l0.728,2.548 c0.832-0.52,2.21-0.962,3.51-0.962c1.95,0,2.314,0.962,2.314,1.639v0.183C2.964,14.365,0,15.952,0,19.253 c0,2.028,1.534,3.9,4.108,3.9c1.508,0,2.808-0.547,3.64-1.562h0.078l0.234,1.274h3.562c-0.13-0.702-0.182-1.872-0.182-3.096 V15.353z M7.592,18.239c0,0.232-0.026,0.468-0.078,0.676c-0.26,0.807-1.066,1.456-2.054,1.456c-0.884,0-1.56-0.494-1.56-1.508 c0-1.534,1.612-2.028,3.692-2.003V18.239z"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M23.062,5.343h-5.201l-5.355,17.524h4.108l1.247-4.498h5.019l1.353,4.498h4.264L23.062,5.343z M18.434,15.404l1.041-3.718 c0.285-1.014,0.545-2.34,0.806-3.38h0.052c0.26,1.04,0.572,2.34,0.884,3.38l1.093,3.718H18.434z"
    }), /*#__PURE__*/React.createElement("polygon", {
      points: "8.025,6.584 1.999,6.584 1.999,6.833 1.999,7.461 1.999,7.709 8.025,7.709 8.025,8.95 12.442,7.146 8.025,5.343"
    }));
  },
  lowerCaseTransform: function lowerCaseTransform(_ref49) {
    var className = _ref49.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      enableBackground: "new 0 0 28.08 28.08",
      fill: "none",
      viewBox: "0 0 28.08 28.08"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M5.356,5.135L0,22.659h4.108l1.248-4.498h5.018l1.352,4.498h4.264L10.556,5.135H5.356z M5.928,15.196l1.04-3.718 c0.286-1.014,0.546-2.34,0.806-3.38h0.052c0.26,1.04,0.572,2.34,0.884,3.38l1.092,3.718H5.928z"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M27.898,19.564v-4.42c0-2.99-1.353-5.486-5.59-5.486c-2.314,0-4.058,0.624-4.94,1.118l0.728,2.548 c0.832-0.52,2.211-0.962,3.51-0.962c1.951,0,2.314,0.962,2.314,1.639v0.183c-4.498-0.026-7.462,1.561-7.462,4.861 c0,2.028,1.534,3.9,4.108,3.9c1.508,0,2.808-0.547,3.64-1.562h0.078l0.233,1.274h3.562C27.95,21.957,27.898,20.787,27.898,19.564z M24.05,18.031c0,0.232-0.026,0.468-0.077,0.676c-0.261,0.807-1.066,1.456-2.055,1.456c-0.884,0-1.561-0.494-1.561-1.508 c0-1.534,1.613-2.028,3.692-2.003V18.031z"
    }), /*#__PURE__*/React.createElement("polygon", {
      points: "21.663,8.742 26.08,6.938 21.663,5.135 21.663,6.376 15.637,6.376 15.637,6.625 15.637,7.253 15.637,7.501 21.663,7.501"
    }));
  },
  cutomInline: function cutomInline(_ref50) {
    var className = _ref50.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 426.66667 426.66667"
    }, /*#__PURE__*/React.createElement("path", {
      d: "m405.332031 192h-170.664062v-170.667969c0-11.773437-9.558594-21.332031-21.335938-21.332031-11.773437 0-21.332031 9.558594-21.332031 21.332031v170.667969h-170.667969c-11.773437 0-21.332031 9.558594-21.332031 21.332031 0 11.777344 9.558594 21.335938 21.332031 21.335938h170.667969v170.664062c0 11.777344 9.558594 21.335938 21.332031 21.335938 11.777344 0 21.335938-9.558594 21.335938-21.335938v-170.664062h170.664062c11.777344 0 21.335938-9.558594 21.335938-21.335938 0-11.773437-9.558594-21.332031-21.335938-21.332031zm0 0"
    }));
  },
  IconCross: function IconCross(_ref51) {
    var className = _ref51.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 409.6 409.6"
    }, /*#__PURE__*/React.createElement("path", {
      d: "m405.332031 192h-170.664062v-170.667969c0-11.773437-9.558594-21.332031-21.335938-21.332031-11.773437 0-21.332031 9.558594-21.332031 21.332031v170.667969h-170.667969c-11.773437 0-21.332031 9.558594-21.332031 21.332031 0 11.777344 9.558594 21.335938 21.332031 21.335938h170.667969v170.664062c0 11.777344 9.558594 21.335938 21.332031 21.335938 11.777344 0 21.335938-9.558594 21.335938-21.335938v-170.664062h170.664062c11.777344 0 21.335938-9.558594 21.335938-21.335938 0-11.773437-9.558594-21.332031-21.335938-21.332031zm0 0"
    }));
  },
  IconMinus: function IconMinus(_ref52) {
    var className = _ref52.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 409.6 409.6"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M392.533,187.733H17.067C7.641,187.733,0,195.374,0,204.8s7.641,17.067,17.067,17.067h375.467 c9.426,0,17.067-7.641,17.067-17.067S401.959,187.733,392.533,187.733z"
    }));
  },
  multipleChoice: function multipleChoice(_ref53) {
    var className = _ref53.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, " Add Multiple Choice "), /*#__PURE__*/React.createElement("path", {
      d: "M16.54,11L13,7.46l1.41-1.41l2.12,2.12l4.24-4.24l1.41,1.41L16.54,11z M11,7H2v2h9V7z M21,13.41L19.59,12L17,14.59 L14.41,12L13,13.41L15.59,16L13,18.59L14.41,20L17,17.41L19.59,20L21,18.59L18.41,16L21,13.41z M11,15H2v2h9V15z"
    }));
  },
  gapQuestion: function gapQuestion(_ref54) {
    var className = _ref54.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, " Add Fill The Gap Question "), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0V0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M21 3H3c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h18c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm0 16H3V5h18v14zM5 15h14v3H5z"
    }));
  },
  insertGap: function insertGap(_ref55) {
    var className = _ref55.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, " Add Gap "), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0V0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M18 9v4H6V9H4v6h16V9h-2z"
    }));
  },
  plusSquare: function plusSquare(_ref56) {
    var className = _ref56.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, " Add Option "), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M13 7h-2v4H7v2h4v4h2v-4h4v-2h-4V7zm-1-5C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm0 18c-4.41 0-8-3.59-8-8s3.59-8 8-8 8 3.59 8 8-3.59 8-8 8z"
    }));
  },
  deleteOutlined: function deleteOutlined(_ref57) {
    var className = _ref57.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, " Delete Option "), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M6 19c0 1.1.9 2 2 2h8c1.1 0 2-.9 2-2V7H6v12zM19 4h-3.5l-1-1h-5l-1 1H5v2h14V4z"
    }), ' ');
  },
  deleteOutlinedQuestion: function deleteOutlinedQuestion(_ref58) {
    var className = _ref58.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, " Delete Question "), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M6 19c0 1.1.9 2 2 2h8c1.1 0 2-.9 2-2V7H6v12zM19 4h-3.5l-1-1h-5l-1 1H5v2h14V4z"
    }), ' ');
  },
  essay: function essay(_ref59) {
    var className = _ref59.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, " Create Essay "), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M14 2H6c-1.1 0-1.99.9-1.99 2L4 20c0 1.1.89 2 1.99 2H18c1.1 0 2-.9 2-2V8l-6-6zm2 16H8v-2h8v2zm0-4H8v-2h8v2zm-3-5V3.5L18.5 9H13z"
    }));
  },
  mulitpleDropDownQuestion: function mulitpleDropDownQuestion(_ref60) {
    var className = _ref60.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, " Create Multiple DropDown Question"), /*#__PURE__*/React.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M19 5v14H5V5h14m0-2H5c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2z"
    }), ' ');
  },
  mulitpleDropDown: function mulitpleDropDown(_ref61) {
    var className = _ref61.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, " Create DropDown "), /*#__PURE__*/React.createElement("path", {
      d: "M17 5H20L18.5 7L17 5M3 2H21C22.11 2 23 2.9 23 4V8C23 9.11 22.11 10 21 10H16V20C16 21.11 15.11 22 14 22H3C1.9 22 1 21.11 1 20V4C1 2.9 1.9 2 3 2M3 4V8H14V4H3M21 8V4H16V8H21M3 20H14V10H3V20M5 12H12V14H5V12M5 16H12V18H5V16Z"
    }));
  },
  refresh: function refresh(_ref62) {
    var className = _ref62.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, " Refresh List"), /*#__PURE__*/React.createElement("path", {
      d: "M12 2A10 10 0 1 0 22 12A10 10 0 0 0 12 2M18 11H13L14.81 9.19A3.94 3.94 0 0 0 12 8A4 4 0 1 0 15.86 13H17.91A6 6 0 1 1 12 6A5.91 5.91 0 0 1 16.22 7.78L18 6Z"
    }), ' ');
  },
  expand: function expand(_ref63) {
    var className = _ref63.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      fill: "none",
      viewBox: "0 0 24 24"
    }, /*#__PURE__*/React.createElement("title", null, ' ', /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Expand",
      label: "Wax.Various.Expand"
    })), /*#__PURE__*/React.createElement("path", {
      d: "M7 10l5 5 5-5z"
    }), ' ');
  },
  help: function help(_ref64) {
    var className = _ref64.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 -960 960 960"
    }, /*#__PURE__*/React.createElement("title", null, " help"), /*#__PURE__*/React.createElement("path", {
      d: "M478-240q21 0 35.5-14.5T528-290q0-21-14.5-35.5T478-340q-21 0-35.5 14.5T428-290q0 21 14.5 35.5T478-240Zm-36-154h74q0-33 7.5-52t42.5-52q26-26 41-49.5t15-56.5q0-56-41-86t-97-30q-57 0-92.5 30T342-618l66 26q5-18 22.5-39t53.5-21q32 0 48 17.5t16 38.5q0 20-12 37.5T506-526q-44 39-54 59t-10 73Zm38 314q-83 0-156-31.5T197-197q-54-54-85.5-127T80-480q0-83 31.5-156T197-763q54-54 127-85.5T480-880q83 0 156 31.5T763-763q54 54 85.5 127T880-480q0 83-31.5 156T763-197q-54 54-127 85.5T480-80Zm0-80q134 0 227-93t93-227q0-134-93-227t-227-93q-134 0-227 93t-93 227q0 134 93 227t227 93Zm0-320Z"
    }));
  },
  deleteIco: function deleteIco(_ref65) {
    var className = _ref65.className;
    return /*#__PURE__*/React.createElement("svg", {
      className: className,
      fill: "none",
      height: "13",
      viewBox: "0 0 20 20",
      width: "20",
      xmlns: "http://www.w3.org/2000/svg"
    }, /*#__PURE__*/React.createElement("title", null, /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Discard",
      label: "Wax.Various.Discard"
    })), /*#__PURE__*/React.createElement("path", {
      d: "M1.65222 5.47827H18.3479",
      stroke: "#FF4E4E",
      strokeLinecap: "round"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M5.82666 5.47826V3C5.82666 2.44771 6.27438 2 6.82666 2H13.1745C13.7268 2 14.1745 2.44772 14.1745 3V5.47826",
      stroke: "#FF4E4E",
      strokeLinecap: "round",
      strokeLinejoin: "round"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M3.73889 5.47827L4.38207 17.0555C4.41151 17.5854 4.8498 18 5.38053 18H14.619C15.1497 18 15.588 17.5854 15.6174 17.0555L16.2606 5.47827",
      stroke: "#FF4E4E",
      strokeLinecap: "round",
      strokeLinejoin: "round"
    }));
  },
  tryAgain: function tryAgain(_ref66) {
    var className = _ref66.className;
    return /*#__PURE__*/React.createElement("svg", {
      className: className,
      fill: "none",
      height: "13",
      viewBox: "0 0 20 20",
      width: "20",
      xmlns: "http://www.w3.org/2000/svg"
    }, /*#__PURE__*/React.createElement("title", null, /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Try Again",
      label: "Wax.AI.Try again"
    })), /*#__PURE__*/React.createElement("path", {
      d: "M16.2963 13.0625C16.7471 12.1375 17 11.0983 17 10C17 6.13401 13.866 3 10 3C6.13401 3 3 6.13401 3 10C3 13.866 6.13401 17 10 17C10.8587 17 11.6812 16.8454 12.4414 16.5625",
      stroke: "#595959"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M18 14L15 12V15L18 14Z",
      fill: "#595959",
      stroke: "#595959"
    }));
  },
  insertIco: function insertIco(_ref67) {
    var className = _ref67.className;
    return /*#__PURE__*/React.createElement("svg", {
      className: className,
      fill: "none",
      height: "13",
      viewBox: "0 0 20 20",
      width: "20",
      xmlns: "http://www.w3.org/2000/svg"
    }, /*#__PURE__*/React.createElement("title", null, /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Insert",
      label: "Wax.AI.Insert"
    })), /*#__PURE__*/React.createElement("path", {
      d: "M4 4H16",
      stroke: "#505050"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M4 7H16",
      stroke: "#505050"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M7 10L16 10",
      stroke: "#505050"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M10 13L16 13",
      stroke: "#505050"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M4 9.5V13H7.5",
      stroke: "#505050"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M6 11L8 13L6 15",
      stroke: "#505050"
    }));
  },
  replaceIco: function replaceIco(_ref68) {
    var className = _ref68.className;
    return /*#__PURE__*/React.createElement("svg", {
      className: className,
      fill: "none",
      height: "13",
      viewBox: "0 0 20 20",
      width: "20",
      xmlns: "http://www.w3.org/2000/svg"
    }, /*#__PURE__*/React.createElement("title", null, /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Replace selected text",
      label: "Wax.AI.Replace selected text"
    })), /*#__PURE__*/React.createElement("path", {
      d: "M18 8H2L6 4",
      stroke: "#595959",
      strokeLinejoin: "round"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M2 12H18L14 16",
      stroke: "#595959",
      strokeLinejoin: "round"
    }));
  },
  submitIco: function submitIco(_ref69) {
    var className = _ref69.className;
    return /*#__PURE__*/React.createElement("svg", {
      className: className,
      fill: "none",
      height: "12",
      viewBox: "0 0 13 12",
      width: "13",
      xmlns: "http://www.w3.org/2000/svg"
    }, /*#__PURE__*/React.createElement("title", null, /*#__PURE__*/React.createElement(Translation, {
      defaultTrans: "Submit",
      label: "Wax.AI.Submit"
    })), /*#__PURE__*/React.createElement("path", {
      d: "M0 11H8V1",
      stroke: "#434343"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M3.5 4L8 1L12 4",
      stroke: "#434343"
    }));
  },
  loaderIco: function loaderIco(_ref70) {
    var className = _ref70.className;
    return /*#__PURE__*/React.createElement("svg", {
      className: className,
      height: "17",
      viewBox: "0 0 100 100",
      width: "17",
      xmlns: "http://www.w3.org/2000/svg"
    }, /*#__PURE__*/React.createElement("circle", {
      cx: "50",
      cy: "50",
      fill: "none",
      r: "40",
      stroke: "#434343",
      strokeWidth: "10"
    }, /*#__PURE__*/React.createElement("animate", {
      attributeName: "stroke-dasharray",
      dur: "1.5s",
      repeatCount: "indefinite",
      values: "0,150;120,150"
    }), /*#__PURE__*/React.createElement("animate", {
      attributeName: "stroke-dashoffset",
      dur: "1.5s",
      repeatCount: "indefinite",
      values: "0;-120"
    })));
  },
  ai: function ai(_ref71) {
    var className = _ref71.className;
    return /*#__PURE__*/React.createElement(Svg, {
      className: className,
      viewBox: "0 0 100 100"
    }, /*#__PURE__*/React.createElement("path", {
      d: "m91.09,24.24l-6.11-6.11c-.5-.49-1.15-.74-1.8-.74s-1.3.25-1.8.74L15.45,84.07c-1,.99-1,2.61,0,3.61l6.1,6.1c.49.49,1.14.75,1.8.75s1.3-.26,1.8-.75l54.36-54.37,11.58-11.56c.99-1,.99-2.62,0-3.61Zm-1.6,2.01l-9.98,9.97-6.51-6.51,9.97-9.98c.07-.05.13-.08.21-.08s.15.03.2.08l6.11,6.11s.08.12.08.2-.03.15-.08.21Z"
    }), /*#__PURE__*/React.createElement("path", {
      d: "m21.71,45.16l3.54,8.23c.28.65,1.21.66,1.49,0l3.66-8.38c.88-2.03,2.5-3.65,4.53-4.53l8.39-3.66c.65-.28.65-1.21,0-1.49l-8.39-3.66c-2.03-.88-3.65-2.5-4.53-4.53l-3.66-8.39c-.28-.65-1.21-.65-1.49,0l-3.66,8.39c-.88,2.03-2.5,3.65-4.53,4.53l-8.4,3.67c-.65.28-.65,1.2,0,1.49l8.55,3.78c2.02.89,3.63,2.52,4.51,4.55Z"
    }), /*#__PURE__*/React.createElement("path", {
      d: "m42.85,15.33l4.54,2.01c1.07.47,1.93,1.34,2.39,2.42l1.88,4.37c.15.35.64.35.79,0l1.94-4.45c.47-1.08,1.33-1.94,2.41-2.41l4.45-1.94c.35-.15.35-.64,0-.79l-4.45-1.94c-1.08-.47-1.94-1.33-2.41-2.41l-1.94-4.45c-.15-.35-.64-.35-.79,0l-1.94,4.45c-.47,1.08-1.33,1.94-2.41,2.41l-4.46,1.95c-.34.15-.35.64,0,.79Z"
    }), /*#__PURE__*/React.createElement("path", {
      d: "m83.44,72.27l-4.73-2.06c-1.14-.5-2.05-1.41-2.55-2.55l-2.06-4.73c-.16-.37-.68-.37-.84,0l-2.06,4.73c-.5,1.14-1.41,2.05-2.55,2.55l-4.73,2.07c-.37.16-.37.68,0,.84l4.82,2.13c1.14.5,2.05,1.42,2.54,2.57l2,4.64c.16.37.68.37.84,0l2.06-4.72c.5-1.14,1.41-2.05,2.55-2.55l4.73-2.06c.37-.16.37-.68,0-.84Z"
    }));
  },
  settings: function settings(_ref72) {
    var className = _ref72.className;
    return /*#__PURE__*/React.createElement("svg", {
      className: className,
      height: "28px",
      preserveAspectRatio: "none",
      version: "1.1",
      viewBox: "0 0 28 28",
      width: "28px",
      x: "0px",
      xmlns: "http://www.w3.org/2000/svg",
      y: "0px"
    }, /*#__PURE__*/React.createElement("defs", null, /*#__PURE__*/React.createElement("g", {
      id: "icon-Layer0_0_FILL"
    }, /*#__PURE__*/React.createElement("path", {
      d: "\nM 21.3 8.8\nQ 21.75 8.35 21.25 7.85\nL 20.2 6.8\nQ 19.7 6.3 19.25 6.8\nL 18.05 7.95\nQ 16.85 7.15 15.4 6.9\nL 15.4 5.15\nQ 15.4 4.5 14.75 4.5\nL 13.25 4.5\nQ 12.55 4.5 12.55 5.15\nL 12.55 6.9\nQ 11.1 7.15 9.9 8\nL 8.65 6.75\nQ 8.15 6.3 7.7 6.75\nL 6.65 7.8\nQ 6.15 8.3 6.6 8.8\nL 7.9 10.1\nQ 7.1 11.25 6.85 12.65\nL 5.05 12.65\nQ 4.35 12.65 4.35 13.35\nL 4.35 14.8\nQ 4.35 15.5 5.05 15.5\nL 6.85 15.5\nQ 7.1 16.95 7.9 18.1\nL 6.65 19.35\nQ 6.2 19.85 6.65 20.3\nL 7.7 21.35\nQ 8.25 21.9 8.7 21.4\nL 10 20.15\nQ 11.15 20.95 12.55 21.2\nL 12.55 22.95\nQ 12.55 23.6 13.25 23.6\nL 14.75 23.6\nQ 15.4 23.6 15.4 22.95\nL 15.4 21.2\nQ 16.8 20.95 18 20.2\nL 19.2 21.35\nQ 19.65 21.85 20.15 21.35\nL 21.2 20.3\nQ 21.7 19.8 21.2 19.35\nL 20.05 18.15\nQ 20.9 16.95 21.15 15.5\nL 22.8 15.5\nQ 23.5 15.5 23.5 14.8\nL 23.5 13.35\nQ 23.5 12.65 22.8 12.65\nL 21.2 12.65\nQ 20.9 11.25 20.1 10.05\nL 21.3 8.8\nM 10.45 10.45\nQ 11.95 9 14 9 16.05 9 17.5 10.45 19 11.95 19 14 19 16.05 17.5 17.5 16.05 19 14 19 11.95 19 10.45 17.5 9 16.05 9 14 9 11.95 10.45 10.45 Z",
      fill: "var(--color-blue)",
      stroke: "var(--color-fill-stroke)"
    })), /*#__PURE__*/React.createElement("path", {
      d: "\nM 13.144646453857423 2.0080963134765626\nL 15.023285675048829 2.0080963134765626\nQ 15.83736267089844 2.0080963134765626 15.83736267089844 2.8253768920898445\nL 15.83736267089844 5.025747680664063\nQ 17.6533805847168 5.340086364746094 19.156291961669922 6.345970153808594\nL 20.659203338623048 4.900012207031249\nQ 21.22279510498047 4.271334838867187 21.84900817871094 4.900012207031249\nL 23.164055633544923 6.22023468017578\nQ 23.79026870727539 6.848912048339843 23.22667694091797 7.414721679687501\nL 21.723765563964847 8.986415100097657\nQ 22.725706481933592 10.495240783691406 23.101434326171876 12.255537414550782\nL 25.105316162109375 12.255537414550782\nQ 25.982014465332032 12.255537414550782 25.982014465332032 13.135685729980468\nL 25.982014465332032 14.958850097656251\nQ 25.982014465332032 15.838998413085937 25.105316162109375 15.838998413085937\nL 23.038813018798827 15.838998413085937\nQ 22.725706481933592 17.66216278076172 21.661144256591797 19.170988464355467\nL 23.101434326171876 20.679814147949223\nQ 23.727647399902345 21.245623779296878 23.101434326171876 21.87430114746094\nL 21.78638687133789 23.194523620605473\nQ 21.16017379760742 23.823200988769536 20.59658203125 23.194523620605473\nL 19.093670654296876 21.748565673828125\nQ 17.59075927734375 22.69158172607422 15.83736267089844 23.00592041015625\nL 15.83736267089844 25.20629119873047\nQ 15.83736267089844 26.023571777343754 15.023285675048829 26.023571777343754\nL 13.144646453857423 26.023571777343754\nQ 12.267948150634766 26.023571777343754 12.267948150634766 25.20629119873047\nL 12.267948150634766 23.00592041015625\nQ 10.514551544189452 22.69158172607422 9.074261474609376 21.685697937011717\nL 7.446107482910155 23.257391357421874\nQ 6.882515716552734 23.886068725585936 6.193681335449219 23.194523620605473\nL 4.878633880615234 21.87430114746094\nQ 4.315042114257812 21.308491516113286 4.878633880615234 20.679814147949223\nL 6.444166564941406 19.108120727539067\nQ 5.442225646972656 17.66216278076172 5.129119110107422 15.838998413085937\nL 2.8747520446777344 15.838998413085937\nQ 1.9980537414550774 15.838998413085937 1.9980537414550774 14.958850097656251\nL 1.9980537414550774 13.135685729980468\nQ 1.9980537414550774 12.255537414550782 2.8747520446777344 12.255537414550782\nL 5.129119110107422 12.255537414550782\nQ 5.442225646972656 10.495240783691406 6.444166564941406 9.049282836914061\nL 4.816012573242188 7.414721679687501\nQ 4.252420806884766 6.786044311523439 4.878633880615234 6.157366943359374\nL 6.193681335449219 4.837144470214843\nQ 6.75727310180664 4.271334838867187 7.383486175537109 4.837144470214843\nL 8.94901885986328 6.408837890625\nQ 10.451930236816406 5.340086364746094 12.267948150634766 5.025747680664063\nL 12.267948150634766 2.8253768920898445\nQ 12.267948150634766 2.0080963134765626 13.144646453857423 2.0080963134765626 Z\nM 20.346096801757813 13.95296630859375\nQ 20.346096801757813 16.53054351806641 18.467457580566407 18.35370788574219 16.651439666748047 20.239739990234376 14.083966064453126 20.239739990234376 11.516492462158201 20.239739990234376 9.637853240966795 18.35370788574219 7.821835327148437 16.53054351806641 7.821835327148437 13.95296630859375 7.821835327148437 11.375389099121092 9.637853240966795 9.489356994628904 11.516492462158201 7.666192626953125 14.083966064453126 7.666192626953125 16.651439666748047 7.666192626953125 18.467457580566407 9.489356994628904 20.346096801757813 11.375389099121092 20.346096801757813 13.95296630859375 Z",
      fill: "none",
      id: "iconLayer0_0_1_STROKES",
      stroke: "var(--color-stroke)",
      strokeLinecap: "square",
      strokeLinejoin: "miter",
      strokeMiterlimit: "3",
      strokeWidth: "1"
    })), /*#__PURE__*/React.createElement("g", {
      transform: "matrix( 1.2524261474609375, 0, 0, 1.257354736328125, -3.45,-3.65) "
    }, /*#__PURE__*/React.createElement("use", {
      xlinkHref: "#icon-Layer0_0_FILL"
    }), /*#__PURE__*/React.createElement("use", {
      transform: "matrix( 0.7984466552734375, 0, 0, 0.795318603515625, 2.75,2.9) ",
      xlinkHref: "#iconLayer0_0_1_STROKES"
    })));
  },
  send: function send(_ref73) {
    var className = _ref73.className;
    return /*#__PURE__*/React.createElement("svg", {
      className: className,
      height: "18px",
      preserveAspectRatio: "none",
      version: "1.1",
      viewBox: "0 0 18 18",
      width: "18px",
      x: "0px",
      xmlns: "http://www.w3.org/2000/svg",
      y: "0px"
    }, /*#__PURE__*/React.createElement("defs", null, /*#__PURE__*/React.createElement("g", {
      id: "sendicon-Layer0_0_FILL_2"
    }, /*#__PURE__*/React.createElement("path", {
      d: "\nM 6.65 9.9\nL 5.65 14.6 15.4 9.4 5.65 4.2 6.65 8.9 9.25 9.4 6.65 9.9 Z",
      fill: "var(--ai-tool-icon-color, #777)",
      stroke: "none"
    }))), /*#__PURE__*/React.createElement("g", {
      transform: "matrix( 1, 0, 0, 1, 0,0) "
    }, /*#__PURE__*/React.createElement("use", {
      xlinkHref: "#sendicon-Layer0_0_FILL_2"
    })));
  },
  copy: function copy(_ref74) {
    var className = _ref74.className;
    return /*#__PURE__*/React.createElement("svg", {
      className: className,
      id: "copy",
      viewBox: "0 0 50 50",
      xmlns: "http://www.w3.org/2000/svg"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M15.243 19.194a1 1 0 1 0 0-2h-3.77a1 1 0 0 0-1 1v21.951a1 1 0 0 0 1 1h21.951a1 1 0 0 0 1-1V36.38a1 1 0 1 0-2 0v2.765h-19.95V19.194h2.769z"
    }), /*#__PURE__*/React.createElement("path", {
      d: "M41.474 9.146H19.522a1 1 0 0 0-1 1v21.951a1 1 0 0 0 1 1h21.951a1 1 0 0 0 1-1V10.146a.998.998 0 0 0-.999-1zm-1 21.951H20.522V11.146h19.951v19.951z"
    }));
  },
  AskKb: function AskKb(_ref75) {
    var className = _ref75.className;
    return /*#__PURE__*/React.createElement("svg", {
      className: className,
      fill: "#000000",
      version: "1.1",
      viewBox: "0 0 147.923 147.923",
      xmlns: "http://www.w3.org/2000/svg",
      xmlSpace: "preserve"
    }, /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement("path", {
      d: "M92.336,116.398c1.979,0.609,4.007,1.072,6.065,1.377c-3.021,8.427-11.016,14.516-20.46,14.516H21.824 C9.789,132.291,0,122.5,0,110.468V41.88c0-12.035,9.789-21.824,21.824-21.824h16.797C40.149,9.513,49.164,1.35,60.127,1.35h30.279 c12.032,0,21.823,9.789,21.823,21.824V38.04c-2.046-0.42-4.129-0.688-6.235-0.779V23.174c0-8.592-6.99-15.588-15.588-15.588H60.127 c-7.526,0-13.825,5.362-15.271,12.471h33.091c10.528,0,19.333,7.496,21.373,17.421c-2.058,0.25-4.086,0.701-6.107,1.267 c-1.449-7.097-7.745-12.453-15.259-12.453H21.836c-8.592,0-15.588,6.997-15.588,15.589v68.588c0,8.586,6.997,15.589,15.588,15.589 h56.118C84.433,126.05,89.998,122.049,92.336,116.398z M123.05,110.322c-17.993,10.388-41.077,4.201-51.462-13.786 C61.199,78.555,67.389,55.468,85.37,45.085c17.987-10.382,41.077-4.205,51.466,13.786 C147.224,76.855,141.025,99.934,123.05,110.322z M119.933,104.927c15.01-8.659,20.161-27.925,11.496-42.938 c-8.659-15.016-27.925-20.18-42.94-11.512c-15.013,8.668-20.168,27.925-11.503,42.94S104.916,113.592,119.933,104.927z M132.513,108.008l-16.21,9.354l12.471,21.592l16.21-9.354L132.513,108.008z M146.664,132.535l-16.204,9.353 c2.582,4.481,8.306,6.017,12.775,3.423C147.711,142.729,149.258,137.011,146.664,132.535z M60.201,42.646H18.7v4.677h41.5V42.646z M50.656,63.436H18.7v4.676h31.956V63.436z M18.7,88.895h31.956v-4.677H18.7V88.895z M18.7,109.676h41.5V105H18.7V109.676z M98.133,84.188l-9.682-10.727l-4.628,4.187l13.695,15.149l26.755-22.283l-3.994-4.789L98.133,84.188z"
    })));
  },
  CustomPromptsOn: function CustomPromptsOn(_ref76) {
    var className = _ref76.className;
    return /*#__PURE__*/React.createElement("svg", {
      className: className,
      fill: "#000000",
      height: "13px",
      id: "Layer_1",
      version: "1.1",
      viewBox: "0 0 330 330",
      width: "13px",
      xmlns: "http://www.w3.org/2000/svg",
      xmlSpace: "preserve"
    }, /*#__PURE__*/React.createElement("path", {
      d: "M325.607,79.393c-5.857-5.857-15.355-5.858-21.213,0.001l-139.39,139.393L25.607,79.393  c-5.857-5.857-15.355-5.858-21.213,0.001c-5.858,5.858-5.858,15.355,0,21.213l150.004,150c2.813,2.813,6.628,4.393,10.606,4.393  s7.794-1.581,10.606-4.394l149.996-150C331.465,94.749,331.465,85.251,325.607,79.393z",
      id: "XMLID_225_"
    }));
  }
};

function _templateObject4() {
  var data = _taggedTemplateLiteral(["\n  margin: 0 8px;\n  ", "\n"]);

  _templateObject4 = function _templateObject4() {
    return data;
  };

  return data;
}

function _templateObject3() {
  var data = _taggedTemplateLiteral(["\n  background: none;\n  border: none;\n  border-radius: 2px;\n  color: ", ";\n  cursor: pointer;\n  font-family: ", ";\n  font-size: ", ";\n  height: 28px;\n  outline: none;\n  padding: 2px;\n  transition: all 0.1s ease-in;\n  user-select: none;\n\n  > svg {\n    transition: all 0.1s ease-in;\n  }\n\n  &:hover {\n    background: ", ";\n  }\n\n  /* stylelint-disable-next-line order/properties-alphabetical-order */\n  ", "\n  ", "\n\n  ", "\n"]);

  _templateObject3 = function _templateObject3() {
    return data;
  };

  return data;
}

function _templateObject2() {
  var data = _taggedTemplateLiteral(["\n  background: ", ";\n  color: ", ";\n\n  > svg {\n    fill: ", ";\n  }\n\n  &:hover {\n    background: ", ";\n  }\n"]);

  _templateObject2 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$1() {
  var data = _taggedTemplateLiteral(["\n  background: ", ";\n  cursor: not-allowed;\n  opacity: 0.4;\n\n  &:hover {\n    background: ", ";\n  }\n"]);

  _templateObject$1 = function _templateObject() {
    return data;
  };

  return data;
}
var disabledStyles = css(_templateObject$1(), function (props) {
  return props.active ? "".concat(th('colorPrimary')) : 'transparent';
}, function (props) {
  return props.active ? "".concat(th('colorPrimary')) : 'transparent';
});
var activeStyles = css(_templateObject2(), th('colorPrimary'), th('colorTextReverse'), th('colorTextReverse'), th('colorPrimary'));
var Wrapper = styled.button.attrs(function (props) {
  return {
    title: props.title,
    type: 'button'
  };
})(_templateObject3(), th('colorText'), th('fontInterface'), th('fontSizeBase'), th('colorBackgroundHue'), function (props) {
  return props.active && activeStyles;
}, function (props) {
  return props.disabled && disabledStyles;
}, override('Wax.MenuButton')); // const StyledIcon = styled(Icon)`
//   height: 24px;
//   width: 24px;
// `;

var Label = styled.span(_templateObject4(), function (props) {
  return props.hasIcon && "margin: 0 8px 0 4px;";
});

var MenuButton = function MenuButton(props) {
  var active = props.active,
      className = props.className,
      disabled = props.disabled,
      iconName = props.iconName,
      label = props.label,
      title = props.title,
      onMouseDown = props.onMouseDown;
  var Component = icons[iconName];

  if (typeof iconName === 'function') {
    Component = iconName;
  }

  return /*#__PURE__*/React.createElement(Wrapper, {
    active: active,
    "aria-pressed": active || false,
    className: className,
    disabled: disabled,
    onMouseDown: onMouseDown,
    title: title
  }, iconName && /*#__PURE__*/React.createElement(Component, {
    className: className
  }), label && /*#__PURE__*/React.createElement(Label, {
    hasIcon: !!iconName
  }, label));
};

MenuButton.defaultProps = {
  iconName: null,
  label: null,
  title: null
};

function _templateObject2$1() {
  var data = _taggedTemplateLiteral(["\n  background: white;\n  margin-top: ", ";\n  position: absolute;\n\n  ", "\n"]);

  _templateObject2$1 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$2() {
  var data = _taggedTemplateLiteral(["\n  font-size: 0;\n  position: relative;\n  z-index: 2;\n"]);

  _templateObject$2 = function _templateObject() {
    return data;
  };

  return data;
}

var Wrapper$1 = styled.div(_templateObject$2());
var DropWrapper = styled.div(_templateObject2$1(), grid(1), override('Wax.MoreDropWrapper'));

var Dropdown = function Dropdown(props) {
  var className = props.className,
      disabled = props.disabled,
      dropComponent = props.dropComponent,
      iconName = props.iconName,
      label = props.label,
      title = props.title;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      isOpen = _useState2[0],
      setIsOpen = _useState2[1];

  var _useTranslation = useTranslation(),
      t = _useTranslation.t,
      i18n = _useTranslation.i18n;

  return /*#__PURE__*/React.createElement(Wrapper$1, {
    className: className
  }, /*#__PURE__*/React.createElement(MenuButton, {
    active: isOpen,
    disabled: disabled,
    iconName: iconName,
    label: !isEmpty(i18n) && i18n.exists("Wax.Various.".concat(label)) ? t("Wax.Various.".concat(label)) : label,
    onMouseDown: function onMouseDown(event) {
      event.preventDefault();
      setIsOpen(!isOpen);
    },
    title: !isEmpty(i18n) && i18n.exists("Wax.Various.".concat(title)) ? t("Wax.Various.".concat(title)) : title
  }), isOpen && /*#__PURE__*/React.createElement(DropWrapper, null, dropComponent));
};

Dropdown.defaultProps = {
  disabled: false,
  iconName: null,
  label: null,
  title: null
};

function _templateObject2$2() {
  var data = _taggedTemplateLiteral(["\n  border: 1px solid gray;\n  padding: 4px;\n"]);

  _templateObject2$2 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$3() {
  var data = _taggedTemplateLiteral(["\n  align-items: center;\n  display: inline-flex;\n  padding: 0 4px;\n\n  > button,\n  > div {\n    margin: 0 5px;\n  }\n"]);

  _templateObject$3 = function _templateObject() {
    return data;
  };

  return data;
}
var Wrapper$2 = styled.div(_templateObject$3());
var DropWrapper$1 = styled(Wrapper$2)(_templateObject2$2());

var ToolGroupComponent = function ToolGroupComponent(_ref) {
  var view = _ref.view,
      tools = _ref.tools,
      name = _ref.name;
  var toolsShown = [];
  var rest = [];
  tools.forEach(function (tool) {
    tool.isIntoMoreSection() && tool.isDisplayed() ? rest.push(tool.renderTool(view)) : toolsShown.push(tool.renderTool(view));
  });
  var MemorizedToolGroupComponent = useMemo(function () {
    return /*#__PURE__*/React.createElement(Wrapper$2, {
      "data-name": name
    }, toolsShown, rest.length > 0 && /*#__PURE__*/React.createElement(Dropdown, {
      dropComponent: /*#__PURE__*/React.createElement(DropWrapper$1, null, rest),
      iconName: "more",
      title: "Show more tools"
    }));
  }, []);
  return MemorizedToolGroupComponent;
};

/* eslint no-underscore-dangle: 0 */

var ToolGroups = function ToolGroups(_ref) {
  var toolGroups = _ref.toolGroups,
      view = _ref.view;
  return toolGroups.map(function (toolGroup) {
    if (toolGroup._toolGroups.length > 0) {
      return /*#__PURE__*/React.createElement(ToolGroups, {
        toolGroups: toolGroup._toolGroups,
        view: view
      });
    }

    return /*#__PURE__*/React.createElement(ToolGroupComponent, {
      key: v4(),
      tools: toolGroup._tools,
      view: view
    });
  });
};

var _dec$6, _class$6, _temp$6;
var ToolGroup = (_dec$6 = injectable(), _dec$6(_class$6 = (_temp$6 = /*#__PURE__*/function () {
  function ToolGroup() {
    _classCallCheck(this, ToolGroup);

    this._config = {};
    this.title = '';
    this._tools = [];
    this._toolGroups = [];
  }

  _createClass(ToolGroup, [{
    key: "setGroupConfig",
    value: function setGroupConfig(config) {
      this._config = config;
    }
  }, {
    key: "excludeIncludeTools",
    value: function excludeIncludeTools() {
      var _this$_config = this._config,
          _this$_config$exclude = _this$_config.exclude,
          exclude = _this$_config$exclude === void 0 ? [] : _this$_config$exclude,
          _this$_config$include = _this$_config.include,
          include = _this$_config$include === void 0 ? [] : _this$_config$include;

      if (include.length > 0) {
        this._tools.map(function (tool) {
          if (include.includes(tool.name)) {
            tool.displayTool();
          } else {
            tool.hideTool();
          }

          return true;
        });
      } else {
        this._tools.map(function (tool) {
          if (exclude.includes(tool.name)) {
            tool.hideTool();
          }
        });
      }
    }
  }, {
    key: "addToolIntoMore",
    value: function addToolIntoMore() {
      var _this$_config$more = this._config.more,
          more = _this$_config$more === void 0 ? [] : _this$_config$more;

      if (more.length > 0) {
        this._tools.map(function (tool) {
          if (more.includes(tool.name)) {
            tool.hideInToolGroup();
          } else {
            tool.displayInToolGroup();
          }
        });
      }
    }
  }, {
    key: "renderTools",
    value: function renderTools(view) {
      var _this = this;

      if (isEmpty(view)) return null;
      var name = this.constructor.name;

      if (this._toolGroups > 0) {
        return /*#__PURE__*/React.createElement(ToolGroups, {
          toolGroups: this._toolGroups,
          view: view
        });
      }

      var MemorizedToolGroupComponent = useMemo(function () {
        return /*#__PURE__*/React.createElement(ToolGroupComponent, {
          key: v4(),
          name: name,
          title: _this.title,
          tools: _this._tools,
          view: view
        });
      }, []);
      return MemorizedToolGroupComponent;
    }
  }, {
    key: "toolGroups",
    set: function set(groups) {
      this._toolGroups = groups;
    }
  }, {
    key: "tools",
    set: function set(tools) {
      this._tools = tools;
    }
  }]);

  return ToolGroup;
}(), _temp$6)) || _class$6);

/* eslint no-underscore-dangle: 0 */

var MainMenuBar = function MainMenuBar(_ref) {
  var _ref$items = _ref.items,
      items = _ref$items === void 0 ? [] : _ref$items,
      view = _ref.view;
  return /*#__PURE__*/React.createElement(React.Fragment, null, map(items, function (item) {
    return item.renderTools(view);
  }));
};

var _dec$7, _class$7, _temp$7;
var Menu = (_dec$7 = injectable(), _dec$7(_class$7 = (_temp$7 = /*#__PURE__*/function () {
  function Menu(config, createTools) {
    _classCallCheck(this, Menu);

    this.toolGroups = [];
    this.config = {};
    this.name = '';
    this.name = config.name;
    this.config = config;
    this.toolGroups = createTools(this.config.toolGroups);
    this.excludeIncludeTools();
    this.addToolIntoMore();
  }

  _createClass(Menu, [{
    key: "excludeIncludeTools",
    value: function excludeIncludeTools() {
      this.toolGroups.forEach(function (toolGroup) {
        if (toolGroup instanceof ToolGroup) {
          toolGroup.excludeIncludeTools();
        }
      });
    }
  }, {
    key: "addToolIntoMore",
    value: function addToolIntoMore() {
      this.toolGroups.forEach(function (toolGroup) {
        if (toolGroup instanceof ToolGroup) {
          toolGroup.addToolIntoMore();
        }
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this = this;

      return function () {
        var _useContext = useContext(WaxContext),
            activeView = _useContext.activeView;

        var Bar = useMemo(function () {
          return (
            /*#__PURE__*/
            // eslint-disable-next-line react/no-this-in-sfc
            React.createElement(MainMenuBar, {
              items: _this.toolGroups,
              view: activeView || {}
            })
          );
        });
        return /*#__PURE__*/React.createElement(React.Fragment, null, Bar);
      };
    }
  }]);

  return Menu;
}(), _temp$7)) || _class$7);

var _dec$8, _class$8, _temp$8;
var MenuCollection = (_dec$8 = injectable(), _dec$8(_class$8 = (_temp$8 = /*#__PURE__*/function () {
  function MenuCollection(menus) {
    _classCallCheck(this, MenuCollection);

    this.menus = [];
    this.menus = menus;
  }

  MenuCollection = multiInject('Menu')(MenuCollection, undefined, 0) || MenuCollection;

  _createClass(MenuCollection, [{
    key: "getMenu",
    value: function getMenu(name) {
      return this.menus.find(function (menu) {
        return menu.name === name;
      });
    }
  }]);

  return MenuCollection;
}(), _temp$8)) || _class$8);

var MenuService = /*#__PURE__*/function (_Service) {
  _inherits(MenuService, _Service);

  var _super = _createSuper(MenuService);

  function MenuService() {
    var _this;

    _classCallCheck(this, MenuService);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.name = 'MenuService';
    return _this;
  }

  _createClass(MenuService, [{
    key: "boot",
    value: function boot() {
      if (this.app.config.get('config.MenuService') === undefined) return false;

      var _this$container$get = this.container.get('MenuCollection'),
          menus = _this$container$get.menus;

      var layout = this.container.get('Layout');
      menus.forEach(function (menu) {
        if (layout.components[menu.config.templateArea]) {
          layout.components[menu.config.templateArea].clear();
        }
      });
      menus.forEach(function (menu) {
        layout.addComponent(menu.config.templateArea, menu.render());
      });
    }
  }, {
    key: "register",
    value: function register() {
      var _this2 = this;

      if (this.app.config.get('config.MenuService') === undefined) return false;
      /* create Menu Factory */

      this.config.map(function (conf) {
        _this2.container.bind('Menu').toFactory(function (context) {
          return new Menu(conf, context.container.get('createTools'));
        });
      });
      /* create MenuCollection of Menus */

      this.container.bind('MenuCollection').to(MenuCollection).inSingletonScope();
      /* create factory of tools */

      this.container.bind('createTools').toFactory(function (context) {
        return function (configTools) {
          var tools = [];
          configTools.forEach(function (tool) {
            try {
              var tl = {};

              if (isPlainObject(tool)) {
                tl = context.container.get(tool.name);
                tl.setGroupConfig(tool);
              } else if (isFunction(tool)) {
                tl = context.container.get(tool());
              } else {
                tl = context.container.get(tool);
              }

              tools.push(tl);
            } catch (error) {
              var service = tool.name ? tool.name : tool;
              throw Error("Could not load Service ".concat(service, ". Please configure service through config"));
            }
          });
          return tools;
        };
      });
    }
  }]);

  return MenuService;
}(Service);

function _templateObject$4() {
  var data = _taggedTemplateLiteral(["\n  left: ", ";\n  position: ", ";\n  top: ", ";\n  z-index: ", ";\n"]);

  _templateObject$4 = function _templateObject() {
    return data;
  };

  return data;
}
var OverlayContainer = styled.div(_templateObject$4(), function (props) {
  return "".concat(props.position.left, "px");
}, function (props) {
  return props.position.position;
}, function (props) {
  return "".concat(props.position.top, "px");
}, function (props) {
  return props.position.zIndex;
});

var Overlay = function Overlay(props) {
  return /*#__PURE__*/React.createElement(OverlayContainer, props, props.children);
};

var defaultOverlay = {
  left: null,
  top: null,
  from: null,
  to: null,
  mark: null
};
var usePosition = (function (options) {
  var _useContext = useContext(WaxContext),
      activeView = _useContext.activeView;

  var _useState = useState(_objectSpread2({
    position: 'absolute',
    zIndex: 999
  }, defaultOverlay)),
      _useState2 = _slicedToArray(_useState, 2),
      position = _useState2[0],
      setPosition = _useState2[1];

  var mark = {};
  var node = '';
  /* Sets Default position at the end of the annotation. You
  can overwrite the default position in your component.
  */

  var calculatePosition = function calculatePosition(focusedView, from, to) {
    var WaxSurface = focusedView.dom.getBoundingClientRect();
    var end = focusedView.coordsAtPos(to);
    var left = end.left - WaxSurface.left;
    var top = end.top - WaxSurface.top + 15;
    return {
      top: top,
      left: left
    };
  };

  var displayOnSelection = function displayOnSelection(focusedView) {
    var selection = focusedView.state.selection;
    var from = selection.from,
        to = selection.to;
    if (from === to) return defaultOverlay;

    var _calculatePosition = calculatePosition(focusedView, from, to),
        left = _calculatePosition.left,
        top = _calculatePosition.top;

    return {
      left: left,
      top: top,
      from: from,
      to: to,
      selection: selection
    };
  };

  var displayOnNode = function displayOnNode(focusedView, overlayOptions) {
    var nodeType = overlayOptions.nodeType,
        followCursor = overlayOptions.followCursor,
        findInParent = overlayOptions.findInParent;
    var PMnode = focusedView.state.schema.nodes[nodeType];
    node = DocumentHelpers.findNode(focusedView.state, PMnode, findInParent);
    if (!isObject(node)) return defaultOverlay;

    var _ref = followCursor ? focusedView.state.selection : node,
        from = _ref.from,
        to = _ref.to;

    var _calculatePosition2 = calculatePosition(focusedView, from, to),
        left = _calculatePosition2.left,
        top = _calculatePosition2.top;

    return {
      left: left,
      top: top,
      from: from,
      to: to,
      node: node
    };
  };

  var displayOnMark = function displayOnMark(focusedView, overlayOptions) {
    var markType = overlayOptions.markType,
        followCursor = overlayOptions.followCursor;
    var PMmark = focusedView.state.schema.marks[markType];
    mark = DocumentHelpers.findMark(focusedView.state, PMmark);
    if (!isObject(mark)) return defaultOverlay;

    var _ref2 = followCursor ? focusedView.state.selection : mark,
        from = _ref2.from,
        to = _ref2.to;

    var _calculatePosition3 = calculatePosition(focusedView, from, to),
        left = _calculatePosition3.left,
        top = _calculatePosition3.top;

    return {
      left: left,
      top: top,
      from: from,
      to: to,
      mark: mark
    };
  };

  var displayOnMarkOrSelection = function displayOnMarkOrSelection(focusedView, overlayOptions) {
    var markType = overlayOptions.markType,
        followCursor = overlayOptions.followCursor;
    var PMmark = focusedView.state.schema.marks[markType];
    mark = DocumentHelpers.findMark(focusedView.state, PMmark);
    if (!isObject(mark)) return displayOnSelection(focusedView);

    var _ref3 = followCursor ? focusedView.state.selection : mark,
        from = _ref3.from,
        to = _ref3.to;

    var _calculatePosition4 = calculatePosition(focusedView, from, to),
        left = _calculatePosition4.left,
        top = _calculatePosition4.top;

    return {
      left: left,
      top: top,
      from: from,
      to: to,
      mark: mark
    };
  }; // eslint-disable-next-line no-unused-vars


  var updatePosition = useCallback(function () {
    if (Object.keys(activeView).length === 0) return defaultOverlay;
    var markType = options.markType,
        selection = options.selection,
        nodeType = options.nodeType;

    if (nodeType) {
      return displayOnNode(activeView, options);
    }

    if (markType && selection) return displayOnMarkOrSelection(activeView, options);
    if (selection) return displayOnSelection(activeView);
    return displayOnMark(activeView, options);
  });
  useLayoutEffect(function () {
    setPosition(_objectSpread2({
      position: 'absolute',
      zIndex: 999
    }, updatePosition(options.followCursor)));
  }, [JSON.stringify(updatePosition(options.followCursor))]);
  return [position, setPosition, mark];
});

var OverlayComponent = (function (Component, markType) {
  return function (props) {
    var context = useContext(WaxContext);

    var _usePosition = usePosition(markType),
        _usePosition2 = _slicedToArray(_usePosition, 3),
        position = _usePosition2[0],
        setPosition = _usePosition2[1],
        mark = _usePosition2[2];

    var component = useMemo(function () {
      return /*#__PURE__*/React.createElement(Component, Object.assign({
        mark: mark,
        position: position,
        setPosition: setPosition
      }, props));
    }, [JSON.stringify(mark), position, context.activeViewId]);
    var visible = position.left !== null;
    return /*#__PURE__*/React.createElement(Overlay, {
      position: position
    }, props.activeViewId === context.activeViewId && visible && component);
  };
});

var OverlayService = /*#__PURE__*/function (_Service) {
  _inherits(OverlayService, _Service);

  var _super = _createSuper(OverlayService);

  function OverlayService() {
    _classCallCheck(this, OverlayService);

    return _super.apply(this, arguments);
  }

  _createClass(OverlayService, [{
    key: "register",
    value: function register() {
      this.container.bind('CreateOverlay').toFactory(function (context) {
        return function (Component, componentProps, options) {
          var layout = context.container.get('Layout');
          layout.addComponent('waxOverlays', OverlayComponent(Component, options), componentProps);
        };
      });
    }
  }]);

  return OverlayService;
}(Service);

var defaultConfig = (function () {
  return {
    services: [new SchemaService(), new RulesService(), new ShortCutsService(), new LayoutService(), new PortalService(), new MenuService(), new OverlayService()]
  };
});

var _dec$9, _class$9, _temp$9;
var PmPlugins = (_dec$9 = injectable(), _dec$9(_class$9 = (_temp$9 = /*#__PURE__*/function () {
  function PmPlugins() {
    _classCallCheck(this, PmPlugins);

    this._plugins = new Map();
  }

  _createClass(PmPlugins, [{
    key: "add",
    value: function add(key, plugin) {
      this._plugins.set(key, plugin);
    }
  }, {
    key: "getAll",
    value: function getAll() {
      return _toConsumableArray(this._plugins.values());
    }
  }, {
    key: "replace",
    value: function replace(key, plugin) {
      this._plugins["delete"](key);

      this.add(key, plugin);
    }
  }, {
    key: "get",
    value: function get(key) {
      return this._plugins.get(key);
    }
  }]);

  return PmPlugins;
}(), _temp$9)) || _class$9);

var Application = /*#__PURE__*/function () {
  function Application(container) {
    _classCallCheck(this, Application);

    this.id = v4();
    this.container = container;
    this.PmPlugins = container.get('PmPlugins');
  }

  _createClass(Application, [{
    key: "registerServices",
    value: function registerServices() {
      var count = 0;

      while (count < this.config.get('config.services').length) {
        var allServices = this.config.get('config.services');
        var service = this.config.get('config.services')[count];
        /*
          set App to every service
          so services can have access to containers and config
          */

        service.setApp(this);

        if (service.dependencies) {
          var servicePos = count;
          allServices.splice.apply(allServices, [servicePos + 1, 0].concat(_toConsumableArray(service.dependencies)));
        }

        if (service.register) {
          service.register();
        }

        count += 1;
      }
    }
  }, {
    key: "setConfig",
    value: function setConfig() {
      this.config = this.container.get('Config');
    }
  }, {
    key: "setContext",
    value: function setContext(context) {
      this.context = context;
    }
  }, {
    key: "bootServices",
    value: function bootServices() {
      var services = this.config.get('config.services');
      services.forEach(function (plugin) {
        if (plugin.boot) {
          plugin.boot();
        }
      });
    }
  }, {
    key: "getPlugins",
    value: function getPlugins() {
      return this.PmPlugins.getAll();
    }
  }, {
    key: "getSchema",
    value: function getSchema() {
      this.schema = this.container.get('Schema');
      return this.schema.getSchema();
    }
  }, {
    key: "getShortCuts",
    value: function getShortCuts() {
      this.shortCuts = this.container.get('ShortCuts');
      this.PmPlugins.add('shortcuts', this.shortCuts.createShortCuts());
    }
  }, {
    key: "getRules",
    value: function getRules() {
      this.rules = this.container.get('Rules');
      this.PmPlugins.add('rules', this.rules.createRules());
    }
  }, {
    key: "resetApp",
    value: function resetApp() {
      this.container = {};
      this.config = {};
      this.PmPlugins = {};
      this.schema = {};
      this.shortCuts = {};
      this.rules = {};
    }
  }], [{
    key: "create",
    value: function create(config) {
      /* Merge Core Config with User Config */
      var appConfig = deepmerge({
        config: defaultConfig()
      }, cloneDeep(config), {
        customMerge: function customMerge(key) {
          if (key === 'services') {
            return function (coreService, configService) {
              return coreService.concat(configService);
            };
          }

          return true;
        }
      });
      /*
      Create Container
      */

      var container = new Container();
      /*
       Set base bindings for the App to work
      */

      container.bind('PmPlugins').to(PmPlugins).inSingletonScope();
      container.bind('Wax').toFactory(function () {
        return new Application(container);
      });
      container.bind('config').toConstantValue(appConfig);
      container.bind('Config').to(Config).inSingletonScope();
      /*
      Start the App
      */

      var app = container.get('Wax');
      app.setConfig();

      if (appConfig.config.PmPlugins) {
        appConfig.config.PmPlugins.forEach(function (configPlugin) {
          app.PmPlugins.add(configPlugin.key, configPlugin);
        });
      }

      app.registerServices();
      return app;
    }
  }]);

  return Application;
}();

/* eslint-disable no-param-reassign */

var alterNotesSchema = function alterNotesSchema(schema) {
  var notes = [];
  each(schema.nodes, function (node) {
    if (node.groups.includes('notes')) notes.push(node);
  });

  if (notes.length > 0) {
    notes.forEach(function (note) {
      schema.nodes[note.name].spec.toDOM = function (node) {
        if (node) return [note.name, node.attrs, 0];
        return true;
      };
    });
  }
};

var revertNotesSchema = function revertNotesSchema(schema) {
  var notes = [];
  each(schema.nodes, function (node) {
    if (node.groups.includes('notes')) notes.push(node);
  });

  if (notes.length > 0) {
    notes.forEach(function (note) {
      schema.nodes[note.name].spec.toDOM = function (node) {
        if (node) return [note.name, node.attrs];
        return true;
      };
    });
  }
};

var getDocContent = function getDocContent(schema, serializer, targetFormat, context) {
  var content = '';
  alterNotesSchema(schema);

  if (targetFormat === 'JSON') {
    content = {
      type: 'doc',
      content: context.pmViews.main.state.doc.content.toJSON()
    };
  } else {
    var serialize = serializer(schema);
    content = serialize(context.pmViews.main.state.doc.content);
  }

  revertNotesSchema(schema);
  return content;
};

var saveContent = function saveContent(content, onChange, schema, serializer, targetFormat) {
  alterNotesSchema(schema);

  if (targetFormat === 'JSON') {
    onChange(content);
  } else {
    var serialize = serializer(schema);
    onChange(serialize(content));
  }

  revertNotesSchema(schema);
};

var helpers = {
  getDocContent: getDocContent,
  saveContent: saveContent
};

var serializer = function serializer(schema) {
  var WaxSerializer = DOMSerializer.fromSchema(schema);
  return function (content) {
    var container = document.createElement('article');
    container.appendChild(WaxSerializer.serializeFragment(content));
    return container.innerHTML;
  };
};

var createApplication = function createApplication(props) {
  var application = Application.create(props);
  return application;
};

var createObjectHash = function createObjectHash(obj) {
  var str = stringify(obj);
  return CryptoJS.SHA256(str).toString();
};

var createConfigWithHash = function createConfigWithHash(config) {
  var configHash = createObjectHash(config);
  return configHash;
};

var Wax = forwardRef(function (props, innerViewRef) {
  var config = props.config,
      layout = props.layout,
      onChange = props.onChange,
      targetFormat = props.targetFormat;

  var _useState = useState(),
      _useState2 = _slicedToArray(_useState, 2),
      application = _useState2[0],
      setApplication = _useState2[1];

  var _useState3 = useState(null),
      _useState4 = _slicedToArray(_useState3, 2),
      WaxLayout = _useState4[0],
      setWaxLayout = _useState4[1];

  var configHash = createConfigWithHash(config);
  useEffect(function () {
    var newApplication = createApplication(props);
    setApplication(newApplication);
    var Layout = newApplication.container.get('Layout');
    if (layout) Layout.setLayout(layout);
    setWaxLayout(Layout.layoutComponent);
  }, [configHash]);

  var finalOnChange = function finalOnChange(content) {
    if (!onChange) return;
    var schema = application.schema.schema;
    helpers.saveContent(content, onChange, schema, serializer, targetFormat);
  };

  if (!application || !WaxLayout) return null;
  return /*#__PURE__*/React.createElement(ApplicationProvider, {
    app: application
  }, /*#__PURE__*/React.createElement(WaxProvider, null, /*#__PURE__*/React.createElement(PortalProvider, null, /*#__PURE__*/React.createElement(WaxLayout, Object.assign({}, props, {
    app: application,
    innerViewRef: innerViewRef,
    onChange: finalOnChange || function () {
      return true;
    },
    serializer: serializer,
    TrackChange: application.config.get('config.EnableTrackChangeService') || undefined
  })))));
});
Wax.defaultProps = {
  config: {
    SchemaService: DefaultSchema,
    services: []
  },
  customValues: {}
};

/* eslint-disable react/jsx-props-no-spreading */

var ComponentPlugin = function ComponentPlugin(renderArea) {
  return function (layoutProps) {
    var _useContext = useContext(ApplicationContext),
        app = _useContext.app;

    var inject = (app === null || app === void 0 ? void 0 : app.container.isBound('Layout')) ? {
      instance: app === null || app === void 0 ? void 0 : app.container.get('Layout')
    } : null;
    var components = inject ? inject.instance.render(renderArea) : [];
    return components ? components.map(function (_ref, key) {
      var Component = _ref.component,
          componentProps = _ref.componentProps;
      return /*#__PURE__*/React.createElement(Component, Object.assign({
        key: "".concat(renderArea, "-").concat(key)
      }, layoutProps, componentProps));
    }) : null;
  };
};

var removeNode = function removeNode(tr, node, nodePos, map) {
  var newNodePos = map.map(nodePos);
  var selectionBefore = Selection.findFrom(tr.doc.resolve(newNodePos), -1);
  var start = selectionBefore.$anchor.pos;
  var end = newNodePos + 1;
  var delStep = replaceStep$1(tr.doc, start, end);
  tr.step(delStep);
  var stepMap = delStep.getMap();
  map.appendMap(stepMap);
};

/* eslint-disable prefer-object-spread */

var markDeletion = function markDeletion(tr, from, to, user, date, group, viewId) {
  var deletionMark = tr.doc.type.schema.marks.deletion.create({
    user: user.userId,
    username: user.username,
    style: "color: ".concat(user.userColor.deletion, ";") // date

  });
  var deletionMap = new Mapping();
  tr.doc.nodesBetween(from, to, function (node, pos) {
    if (node.type.name.includes('table')) {
      return;
    }

    if (node.isInline && node.marks.find(function (mark) {
      return mark.type.name === 'insertion' && mark.attrs.user.toString() === user.userId.toString();
    })) {
      var removeStep = new ReplaceStep(deletionMap.map(Math.max(from, pos)), deletionMap.map(Math.min(to, pos + node.nodeSize)), Slice.empty);

      if (!tr.maybeStep(removeStep).failed) {
        deletionMap.appendMap(removeStep.getMap());
      }
    } else if (node.isInline && !node.marks.find(function (mark) {
      return mark.type.name === 'deletion';
    })) {
      tr.addMark(deletionMap.map(Math.max(from, pos)), deletionMap.map(Math.min(to, pos + node.nodeSize)), deletionMark);
    } else if (node.attrs.track && !node.attrs.track.find(function (trackAttr) {
      return trackAttr.type === 'deletion';
    }) && !['bullet_list', 'ordered_list'].includes(node.type.name)) {
      if (node.attrs.track.find(function (trackAttr) {
        return trackAttr.type === 'insertion' && trackAttr.user === user.userId;
      })) {
        var _removeStep;

        if (node.isTextblock && node.nodeSize === 2) {
          return removeNode(tr, node, pos, deletionMap);
        }

        if (node.isTextblock && to < pos + node.nodeSize) {
          var selectionBefore = Selection.findFrom(tr.doc.resolve(pos), -1);

          if (selectionBefore instanceof TextSelection) {
            _removeStep = new ReplaceStep( // deletionMap.map(selectionBefore.$anchor.pos),
            deletionMap.map(Math.min(to, pos + node.nodeSize)), deletionMap.map(to), Slice.empty);
          }
        } else {
          _removeStep = new ReplaceStep(deletionMap.map(Math.max(from, pos)), deletionMap.map(Math.min(to, pos + node.nodeSize)), Slice.empty);
        }

        if (!tr.maybeStep(_removeStep).failed) {
          deletionMap.appendMap(_removeStep.getMap());
        }
      }

      var counter = 2;
      node.content.forEach(function (item, i) {
        item.marks.forEach(function (mark) {
          if (mark.type.name === 'deletion') {
            counter += item.nodeSize;
          }
        });
      });

      if (node.content.size === 0 || counter === node.nodeSize || node.type.name === 'figure') {
        var track = node.attrs.track.slice();
        track.push({
          type: 'deletion',
          user: user.userId,
          username: user.username,
          style: "color: ".concat(user.userColor.deletion, ";"),
          date: date,
          group: group,
          viewid: viewId
        });
        tr.setNodeMarkup(deletionMap.map(pos), null, Object.assign({}, node.attrs, {
          track: track,
          group: group,
          id: v4(),
          viewid: viewId
        }), // Object.assign(node.attrs.track, { track }),
        node.marks);
      }
    }
  });
  return deletionMap;
};

/* eslint-disable no-else-return */

var markInsertion = function markInsertion(tr, from, to, user, date, group, viewId) {
  tr.removeMark(from, to, tr.doc.type.schema.marks.deletion);
  tr.removeMark(from, to, tr.doc.type.schema.marks.insertion);
  var insertionMark = tr.doc.type.schema.marks.insertion.create({
    user: user.userId,
    username: user.username,
    style: "color: ".concat(user.userColor.addition, ";") // date

  });
  tr.addMark(from, to, insertionMark); // Add insertion mark also to block nodes (figures, text blocks) but not table cells/rows and lists.

  tr.doc.nodesBetween(from, to, function (node, pos) {
    if (pos < from || ['bullet_list', 'ordered_list'].includes(node.type.name)) {
      return true;
    } else if (node.isInline || ['table_row', 'table_cell'].includes(node.type.name)) {
      return false;
    }

    if (node.attrs.track) {
      var track = [];
      track.push({
        type: 'insertion',
        user: user.userId,
        username: user.username,
        date: date,
        group: group,
        viewid: viewId
      });
      tr.setNodeMarkup(pos, null, // eslint-disable-next-line prefer-object-spread
      Object.assign({}, node.attrs, {
        track: track,
        group: group,
        id: v4(),
        viewid: viewId
      }), node.marks);
    }

    if (node.type.name === 'table') {
      // A table was inserted. We don't add track marks to elements inside of it.
      return false;
    }
  });
};

var replaceStep = function replaceStep(state, tr, step, newTr, map, doc, user, date, group, viewId, originalStep, originalStepIndex) {
  var deletionMarkSchema = state.schema.marks.deletion;
  var deletionMark = DocumentHelpers.findMark(state, deletionMarkSchema, false);
  var positionTo = deletionMark ? deletionMark.to : step.to;
  var newStep = new ReplaceStep(positionTo, // We insert all the same steps, but with "from"/"to" both set to "to" in order not to delete content. Mapped as needed.
  positionTo, step.slice, step.structure); // We didn't apply the original step in its original place. We adjust the map accordingly.

  var invertStep = originalStep.invert(tr.docs[originalStepIndex]).map(map);
  map.appendMap(invertStep.getMap()); // map.appendMap(step.invert(doc).getMap());

  if (newStep) {
    var trTemp = state.apply(newTr).tr;

    if (trTemp.maybeStep(newStep).failed) {
      return;
    }

    var mappedNewStepTo = newStep.getMap().map(newStep.to);
    markInsertion(trTemp, newStep.from, mappedNewStepTo, user, date, group, viewId); // We condense it down to a single replace step.

    var condensedStep = new ReplaceStep(newStep.from, newStep.to, trTemp.doc.slice(newStep.from, mappedNewStepTo));
    newTr.step(condensedStep);
    var mirrorIndex = map.maps.length - 1;
    map.appendMap(condensedStep.getMap(), mirrorIndex);

    if (!newTr.selection.eq(trTemp.selection)) {
      newTr.setSelection(trTemp.selection);
    }
  }

  if (step.from !== step.to) {
    map.appendMap(markDeletion(newTr, step.from, step.to, user, date, group, viewId));
  }
};

var markWrapping = function markWrapping(tr, pos, oldNode, newNode, user, date, group, viewId) {
  var track = oldNode.attrs.track.slice();
  var blockTrack = track.find(function (track) {
    return track.type === 'block_change';
  });

  if (blockTrack) {
    track = track.filter(function (track) {
      return track !== blockTrack;
    });

    if (blockTrack.before.type !== newNode.type.name || blockTrack.before.attrs.level !== newNode.attrs.level) {
      blockTrack = {
        type: 'block_change',
        user: user.userId,
        username: user.username,
        date: date,
        before: blockTrack.before
      };
      track.push(blockTrack);
    }
  } else {
    blockTrack = {
      type: 'block_change',
      user: user.userId,
      username: user.username,
      date: date,
      before: {
        type: oldNode.type.name,
        attrs: oldNode.attrs
      }
    };

    if (blockTrack.before.attrs.id) {
      delete blockTrack.before.attrs.id;
    }

    if (blockTrack.before.attrs.track) {
      delete blockTrack.before.attrs.track;
    }

    track.push(blockTrack);
  }

  tr.setNodeMarkup(pos, null, Object.assign({}, newNode.attrs, {
    track: track,
    group: group,
    id: v4(),
    viewid: viewId
  }));
};

var replaceAroundStep = function replaceAroundStep(state, tr, step, newTr, map, doc, user, date, group, viewId, originalStep, originalStepIndex) {
  if (step.from === step.gapFrom && step.to === step.gapTo) {
    // wrapped in something
    newTr.step(step);
    var from = step.getMap().map(step.from, -1);
    var to = step.getMap().map(step.gapFrom);
    markInsertion(newTr, from, to, user, date, group, viewId);
  } else if (!step.slice.size || step.slice.content.content.length === 2) {
    var invertStep = originalStep.invert(tr.docs[originalStepIndex]).map(map); // unwrapped from something

    map.appendMap(invertStep.getMap()); // map.appendMap(step.invert(doc).getMap());

    map.appendMap(markDeletion(newTr, step.from, step.gapFrom, user, date, group));
  } else if (step.slice.size === 2 && step.gapFrom - step.from === 1 && step.to - step.gapTo === 1) {
    // Replaced one wrapping with another
    newTr.step(step);
    var oldNode = doc.nodeAt(step.from);

    if (oldNode.attrs.track) {
      markWrapping(newTr, step.from, oldNode, step.slice.content.firstChild, user, date, group, viewId);
    }
  } else {
    newTr.step(step);
    var ranges = [{
      from: step.getMap().map(step.from, -1),
      to: step.getMap().map(step.gapFrom)
    }, {
      from: step.getMap().map(step.gapTo, -1),
      to: step.getMap().map(step.to)
    }];
    ranges.forEach(function (range) {
      return doc.nodesBetween(range.from, range.to, function (node, pos) {
        if (pos < range.from) {
          return true;
        }

        markInsertion(newTr, range.from, range.to, user, date, group, viewId);
      });
    });
  }
};

var addMarkStep = function addMarkStep(state, tr, step, newTr, map, doc, user, date, group, viewId) {
  doc.nodesBetween(step.from, step.to, function (node, pos) {
    if (!node.isInline) {
      return true;
    }

    if (node.marks.find(function (mark) {
      return mark.type.name === 'deletion';
    })) {
      return false;
    }

    newTr.addMark(Math.max(step.from, pos), Math.min(step.to, pos + node.nodeSize), step.mark);

    if (!['comment', 'deletion'].includes(step.mark.type.name) && !node.marks.find(function (mark) {
      return mark.type === step.mark.type;
    })) {
      var formatChangeMark = node.marks.find(function (mark) {
        return mark.type.name === 'format_change';
      });
      var after, before;

      if (formatChangeMark) {
        if (formatChangeMark.attrs.before.includes(step.mark.type.name)) {
          before = formatChangeMark.attrs.before.filter(function (markName) {
            return markName !== step.mark.type.name;
          });
          after = formatChangeMark.attrs.after;
        } else {
          before = formatChangeMark.attrs.before;
          after = formatChangeMark.attrs.after.concat(step.mark.type.name);
        }
      } else {
        before = [];
        after = [step.mark.type.name];
      }

      if (after.length || before.length) {
        // Math.max(step.from, pos),
        // Math.min(step.to, pos + node.nodeSize),
        newTr.addMark(step.from, step.to, state.schema.marks.format_change.create({
          id: v4(),
          user: user.userId,
          username: user.username,
          date: date,
          before: before,
          after: after,
          group: group,
          viewid: viewId
        }));
      } else if (formatChangeMark) {
        newTr.removeMark(Math.max(step.from, pos), Math.min(step.to, pos + node.nodeSize), formatChangeMark);
      }
    }
  });
};

var removeMarkStep = function removeMarkStep(state, tr, step, newTr, map, doc, user, date, group) {
  doc.nodesBetween(step.from, step.to, function (node, pos) {
    if (!node.isInline) {
      return true;
    }

    if (node.marks.find(function (mark) {
      return mark.type.name === 'deletion';
    })) {
      return false;
    }

    newTr.removeMark(Math.max(step.from, pos), Math.min(step.to, pos + node.nodeSize), step.mark);

    if (['em', 'strong', 'underline'].includes(step.mark.type.name) && node.marks.find(function (mark) {
      return mark.type === step.mark.type;
    })) {
      var formatChangeMark = node.marks.find(function (mark) {
        return mark.type.name === 'format_change';
      });
      var after, before;

      if (formatChangeMark) {
        if (formatChangeMark.attrs.after.includes(step.mark.type.name)) {
          after = formatChangeMark.attrs.after.filter(function (markName) {
            return markName !== step.mark.type.name;
          });
          before = formatChangeMark.attrs.before;
        } else {
          after = formatChangeMark.attrs.after;
          before = formatChangeMark.attrs.before.concat(step.mark.type.name);
        }
      } else {
        after = [];
        before = [step.mark.type.name];
      }

      if (after.length || before.length) {
        newTr.addMark(Math.max(step.from, pos), Math.min(step.to, pos + node.nodeSize), state.schema.marks.format_change.create({
          user: user.userId,
          username: user.username,
          date: date,
          before: before,
          after: after,
          id: v4(),
          group: group
        }));
      } else if (formatChangeMark) {
        newTr.removeMark(Math.max(step.from, pos), Math.min(step.to, pos + node.nodeSize), formatChangeMark);
      }
    }
  });
};

var trackedTransaction = function trackedTransaction(tr, state, user, app) {
  var group = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'main';
  var viewId = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 'main';

  // Don't track table operations
  if (!tr.selectionSet) {
    var $pos = state.selection.$anchor;

    for (var depth = $pos.depth; depth > 0; depth -= 1) {
      var node = $pos.node(depth);

      if (node.type.spec.tableRole === 'table') {
        return tr;
      }
    }
  }

  if (!tr.steps.length || tr.meta && !Object.keys(tr.meta).every(function (meta) {
    return ['inputType', 'uiEvent', 'paste', 'pointer', 'imagePlaceHolder$', 'imagePlaceHolder$1'].includes(meta);
  }) || ['AcceptReject', 'Undo', 'Redo'].includes(tr.getMeta('inputType'))) {
    return tr;
  } // const group = tr.getMeta('outsideView') ? tr.getMeta('outsideView') : 'main';


  var newTr = state.tr;
  var map = new Mapping();
  var date = Math.floor(Date.now());
  tr.steps.forEach(function (originalStep, originalStepIndex) {
    var step = originalStep.map(map);
    var doc = newTr.doc;
    if (!step) return;

    switch (step.constructor) {
      case ReplaceStep:
        replaceStep(state, tr, step, newTr, map, doc, user, date, group, viewId, originalStep, originalStepIndex);
        break;

      case ReplaceAroundStep:
        replaceAroundStep(state, tr, step, newTr, map, doc, user, date, group, viewId, originalStep, originalStepIndex);
        break;

      case AddMarkStep:
        addMarkStep(state, tr, step, newTr, map, doc, user, date, group, viewId);
        break;

      case RemoveMarkStep:
        removeMarkStep(state, tr, step, newTr, map, doc, user, date, group);
        break;
    }
  });
  if (tr.getMeta('inputType')) newTr.setMeta(tr.getMeta('inputType'));
  if (tr.getMeta('uiEvent')) newTr.setMeta(tr.getMeta('uiEvent'));

  if (tr.selectionSet) {
    var deletionMarkSchema = state.schema.marks.deletion;
    var deletionMark = DocumentHelpers.findMark(state, deletionMarkSchema, false);

    if (tr.selection instanceof TextSelection && (tr.selection.from < state.selection.from || tr.getMeta('inputType') === 'backwardsDelete')) {
      var caretPos = map.map(tr.selection.from, -1);
      newTr.setSelection(new TextSelection(newTr.doc.resolve(caretPos)));
    } else if (tr.selection.from > state.selection.from && deletionMark) {
      var _caretPos = map.map(deletionMark.to + 1, 1);

      newTr.setSelection(new TextSelection(newTr.doc.resolve(_caretPos)));
    } else {
      newTr.setSelection(tr.selection.map(newTr.doc, map));
    }
  } else if (state.selection.from - tr.selection.from > 1 && tr.selection.$head.depth > 1) {
    var _caretPos2 = map.map(tr.selection.from - 2, -1);

    newTr.setSelection(new TextSelection(newTr.doc.resolve(_caretPos2)));
  } else {
    if (state.selection.from === state.selection.to && tr.selection.$head.depth > 1 && state.selection.from - tr.selection.from === 1) {
      tr.steps.forEach(function (originalStep) {
        var step = originalStep.map(map);
        step.from += 1;
        step.to += 1;
        var doc = newTr.doc;
        replaceStep(state, tr, step, newTr, map, doc, user, date, group);
      });

      var _caretPos3 = map.map(tr.selection.from - 1, -1);

      newTr.setSelection(new TextSelection(newTr.doc.resolve(_caretPos3)));
    } else if (state.selection.from === state.selection.to) {
      var _caretPos4 = map.map(tr.selection.from, -1);

      newTr.setSelection(new TextSelection(newTr.doc.resolve(_caretPos4)));
    }

    var slice = map.slice(newTr.selection.from, newTr.selection.to);
    map.appendMap(slice);
  }

  if (tr.storedMarksSet) newTr.setStoredMarks(tr.storedMarks);
  if (tr.scrolledIntoView) newTr.scrollIntoView();

  if (!isEmpty(tr.meta) && Object.keys(tr.meta).every(function (meta) {
    return meta.includes('imagePlaceHolder');
  })) {
    var imagePlaceholder = app.PmPlugins.get('imagePlaceHolder');
    return newTr.setMeta(imagePlaceholder, {
      remove: {
        id: {}
      }
    });
  }

  return newTr;
};

var placeHolderText = new PluginKey('placeHolderText');
var Placeholder = (function (props) {
  return new Plugin({
    key: placeHolderText,
    props: {
      decorations: function decorations(state) {
        var decorations = [];

        var decorate = function decorate(node, pos) {
          if (node.type.isBlock && node.childCount === 0 && state.doc.content.childCount === 1) {
            decorations.push(Decoration.node(pos, pos + node.nodeSize, {
              "class": 'empty-node',
              'data-content': props.content
            }));
          }
        };

        state.doc.descendants(decorate);
        return DecorationSet.create(state.doc, decorations);
      }
    }
  });
});

/* eslint-disable */
var fakeCursorPlugin = new PluginKey('fakeCursorPlugin');
var FakeCursorPlugin = (function (props) {
  return new Plugin({
    key: fakeCursorPlugin,
    state: {
      init: function init(_, state) {},
      apply: function apply(tr, prev, _, newState) {
        var createDecoration;

        if (tr.steps.length === 0 || tr.steps[0] && tr.steps[0].from === tr.steps[0].to) {
          var widget = document.createElement('span');
          widget.setAttribute('id', 'fake-cursor');

          if (newState.selection.from === newState.selection.to) {
            createDecoration = DecorationSet.create(newState.doc, [Decoration.widget(newState.selection.from, widget, {
              key: 'fakecursor'
            })]);
          }

          setTimeout(function () {
            widget.setAttribute('contenteditable', true);

            if (navigator.userAgent.includes('Firefox') && newState.selection.$from.nodeBefore === null) {
              widget.setAttribute('style', 'visibility:hidden');
            } else {
              widget.setAttribute('style', 'display:none');
            }
          });
        }

        return {
          createDecoration: createDecoration
        };
      }
    },
    props: {
      decorations: function decorations(state) {
        var fakeCursorPluginState = state && fakeCursorPlugin.getState(state);
        if (fakeCursorPluginState) return fakeCursorPluginState.createDecoration;
      },
      handleDOMEvents: {
        focus: function focus(view, event) {
          event.preventDefault();
          var fakeCursor = document.getElementById('fake-cursor');

          if (fakeCursor) {
            if (navigator.userAgent.includes('Firefox') && view.state.selection.$from.nodeBefore == null) {
              fakeCursor.style.visibility = 'hidden';
            } else {
              fakeCursor.style.display = 'none';
            }
          }
        },
        blur: function blur(view, event) {
          event.preventDefault();

          if (view && event.relatedTarget === null) {
            setTimeout(function () {
              view.focus();
            });
          } else {
            var fakeCursor = document.getElementById('fake-cursor');

            if (fakeCursor) {
              if (navigator.userAgent.includes('Firefox') && view.state.selection.$from.nodeBefore === null) {
                fakeCursor.style.visibility = 'visible';
              } else {
                fakeCursor.style.display = 'inline';
              }
            }
          }
        }
      }
    }
  });
});

var defaultPlugins = [dropCursor(), gapCursor(), history(), FakeCursorPlugin()];

var parser = function parser(schema) {
  var WaxParser = DOMParser.fromSchema(schema);
  return function (content) {
    var container = document.createElement('article');
    container.innerHTML = content;
    return WaxParser.parse(container, {
      preserveWhitespace: true
    });
  };
};
var WaxOptions = (function (_ref) {
  var placeholder = _ref.placeholder,
      targetFormat = _ref.targetFormat,
      value = _ref.value,
      schema = _ref.schema,
      plugins = _ref.plugins;
  var finalPlugins = [];

  var createPlaceholder = function createPlaceholder() {
    return Placeholder({
      content: placeholder
    });
  };

  finalPlugins = defaultPlugins.concat([createPlaceholder()].concat(_toConsumableArray(plugins)));
  var WaxOptions = {
    doc: {},
    schema: schema,
    plugins: finalPlugins
  };

  if (targetFormat === 'JSON') {
    var editorContent = value || {
      type: 'doc',
      content: [{
        type: 'paragraph',
        content: [{
          type: 'text',
          text: ' '
        }]
      }]
    };
    WaxOptions.doc = schema.nodeFromJSON(editorContent);
  } else {
    var _editorContent = value || '';

    var parse = parser(schema);
    WaxOptions.doc = parse(_editorContent);
  }

  return WaxOptions;
});

var previousDoc;

var useWaxView = function useWaxView(props) {
  var browserSpellCheck = props.browserSpellCheck,
      customValues = props.customValues,
      readonly = props.readonly,
      user = props.user,
      app = props.app,
      autoFocus = props.autoFocus,
      innerViewRef = props.innerViewRef,
      targetFormat = props.targetFormat,
      serializer = props.serializer,
      scrollMargin = props.scrollMargin,
      scrollThreshold = props.scrollThreshold,
      onChange = props.onChange;
  var view;
  var context = useContext(WaxContext);

  var _useState = useState(null),
      _useState2 = _slicedToArray(_useState, 2),
      WaxView = _useState2[0],
      setWaxView = _useState2[1];

  var _useContext = useContext(PortalContext),
      createPortal = _useContext.createPortal;

  app.setContext(_objectSpread2(_objectSpread2({}, context), {}, {
    createPortal: createPortal
  }));
  var schema = app.getSchema();
  useEffect(function () {
    app.bootServices();
    app.getShortCuts();
    app.getRules();
    var options = WaxOptions(_objectSpread2(_objectSpread2({}, props), {}, {
      schema: schema,
      plugins: app.getPlugins()
    }));
    view = new EditorView(null, {
      editable: function editable() {
        return !readonly;
      },
      customValues: customValues,
      state: EditorState.create(options),
      disallowedTools: [],
      user: user,
      scrollMargin: scrollMargin || 200,
      scrollThreshold: scrollThreshold || 200,
      attributes: {
        spellcheck: browserSpellCheck ? 'true' : 'false'
      }
    });
    view.setProps({
      dispatchTransaction: function dispatchTransaction(transaction) {
        var TrackChange = props.TrackChange;
        var tr = TrackChange && TrackChange.enabled ? trackedTransaction(transaction, view.state, user, app) : transaction;
        previousDoc = view.state.doc;
        var state = view.state.apply(tr);
        view.updateState(state);
        context.setTransaction(transaction);

        if (!transaction.getMeta('outsideView')) {
          context.updateView({}, 'main');
        }

        var docContent = targetFormat === 'JSON' ? state.doc.toJSON() : state.doc.content;
        if (!previousDoc.eq(view.state.doc) || tr.getMeta('forceUpdate')) onChange(docContent);
      }
    });
    setWaxView(_objectSpread2({}, view));
    context.updateView({
      main: view
    }, 'main');
    setTimeout(function () {
      if (autoFocus && view) {
        view.focus();
      }
    }, 500);
  }, [readonly, customValues, app.id]);
  useEffect(function () {
    return function () {
      return view = null;
    };
  }, []);
  useImperativeHandle(innerViewRef, function () {
    return {
      getContent: function getContent() {
        return helpers.getDocContent(schema, serializer, targetFormat, context);
      }
    };
  });
  return WaxView;
};

function _templateObject$5() {
  var data = _taggedTemplateLiteral(["\n  height: 100%;\n  position: relative;\n\n  > div:first-child {\n    height: 100%;\n  }\n"]);

  _templateObject$5 = function _templateObject() {
    return data;
  };

  return data;
}
var EditorContainer = styled.div(_templateObject$5());
var WaxPortals = ComponentPlugin('waxPortals');
var WaxOverlays = ComponentPlugin('waxOverlays');

var WaxView = function WaxView(props) {
  var divRef = useRef(null);
  var main = useWaxView(props);
  var initialize = useCallback(function () {
    if (divRef.current) {
      divRef.current.replaceChildren(main === null || main === void 0 ? void 0 : main.dom);
    }
  }, [main]);
  useEffect(function () {
    initialize();
  }, [initialize]);
  return /*#__PURE__*/React.createElement(EditorContainer, null, /*#__PURE__*/React.createElement("div", {
    ref: divRef
  }), /*#__PURE__*/React.createElement(WaxOverlays, {
    activeViewId: "main",
    group: "main"
  }), /*#__PURE__*/React.createElement(WaxPortals, null));
};

var StateContext = React.createContext({
  state: null
});

var parseFormatList = function parseFormatList(str) {
  if (!str) {
    return [];
  }

  var formatList;

  try {
    formatList = JSON.parse(str);
  } catch (error) {
    return [];
  }

  if (!Array.isArray(formatList)) {
    return [];
  }

  return formatList.filter(function (format) {
    return typeof format === 'string';
  });
};

var parseTracks = function parseTracks(str) {
  if (!str) {
    return [];
  }

  var tracks;

  try {
    tracks = JSON.parse(str);
  } catch (error) {
    return [];
  }

  if (!Array.isArray(tracks)) {
    return [];
  }

  return tracks.filter(function (track) {
    return (// ensure required fields are present
      track.hasOwnProperty('user') && track.hasOwnProperty('username') && track.hasOwnProperty('date')
    );
  });
};

var blockLevelToDOM = function blockLevelToDOM(node) {
  var attrs = node.attrs.track && node.attrs.track.length ? {
    'data-id': node.attrs.id,
    "class": node.attrs["class"],
    'data-track': JSON.stringify(node.attrs.track),
    'data-group': node.attrs.group,
    'data-viewid': node.attrs.viewid
  } : {
    "class": node.attrs["class"]
  };
  return attrs;
};

var SchemaHelpers = {
  parseFormatList: parseFormatList,
  parseTracks: parseTracks,
  blockLevelToDOM: blockLevelToDOM
};

var EditoriaSchema = {
  nodes: {
    doc: {
      content: 'block+'
    },
    text: {
      group: 'inline'
    },
    hard_break: {
      inline: true,
      group: 'inline',
      selectable: false,
      parseDOM: [{
        tag: 'br'
      }],
      toDOM: function toDOM() {
        return ['br'];
      }
    },
    paragraph: {
      group: 'block',
      content: 'inline*',
      attrs: {
        id: {
          "default": ''
        },
        "class": {
          "default": 'paragraph'
        },
        track: {
          "default": []
        },
        group: {
          "default": ''
        },
        viewid: {
          "default": ''
        }
      },
      parseDOM: [{
        tag: 'p.paragraph',
        getAttrs: function getAttrs(dom) {
          return {
            id: dom.dataset.id,
            "class": dom.getAttribute('class'),
            track: SchemaHelpers.parseTracks(dom.dataset.track),
            group: dom.dataset.group,
            viewid: dom.dataset.viewid
          };
        }
      }],
      toDOM: function toDOM(node) {
        var attrs = SchemaHelpers.blockLevelToDOM(node);
        return ['p', attrs, 0];
      }
    }
  },
  marks: {}
};

/* eslint react/prop-types: 0 */

var Button = function Button(_ref) {
  var _ref$view = _ref.view,
      item = _ref.item;

  var _useTranslation = useTranslation(),
      t = _useTranslation.t,
      i18n = _useTranslation.i18n;

  var active = item.active,
      icon = item.icon,
      label = item.label,
      run = item.run,
      select = item.select,
      title = item.title;
  var context = useContext(WaxContext);
  var main = context.pmViews.main,
      activeViewId = context.activeViewId,
      activeView = context.activeView;
  var isEditable = main.props.editable(function (editable) {
    return editable;
  });

  var handleMouseDown = function handleMouseDown(e) {
    e.preventDefault();
    run(activeView.state, activeView.dispatch, activeView, context);
  };

  var isActive = !!(active(activeView.state, activeViewId) && select(activeView.state, activeViewId, activeView));
  var isDisabled = !select(context.activeView.state, context.activeViewId, context.activeView);
  if (!isEditable) isDisabled = true;
  var MenuButtonComponent = useMemo(function () {
    return /*#__PURE__*/React.createElement(MenuButton, {
      active: isActive || false,
      disabled: isDisabled,
      iconName: icon,
      label: label,
      onMouseDown: function onMouseDown(e) {
        return handleMouseDown(e);
      },
      title: !isEmpty(i18n) && i18n.exists("Wax.Annotations.".concat(title)) ? t("Wax.Annotations.".concat(title)) : title
    });
  }, [isActive, isDisabled, activeViewId, t("Wax.Annotations.".concat(title))]);
  return MenuButtonComponent;
};

var _dec$a, _class$a, _temp$a;
var Tools = (_dec$a = injectable(), _dec$a(_class$a = (_temp$a = /*#__PURE__*/function () {
  function Tools(config, pmplugins) {
    _classCallCheck(this, Tools);

    this.title = 'title';
    this._isDisplayed = true;
    this._isHiddenInToolGroup = false;
    this.config = {};
    this.pmplugins = {};
    this.name = 'name';
    this.config = config;
    this.pmplugins = pmplugins;
  }

  Tools = inject('PmPlugins')(Tools, undefined, 1) || Tools;
  Tools = inject('Config')(Tools, undefined, 0) || Tools;

  _createClass(Tools, [{
    key: "select",
    value: function select() {
      return function () {
        return true;
      };
    }
  }, {
    key: "toJSON",
    value: function toJSON() {
      return {
        title: this.title,
        icon: this.icon,
        label: this.label,
        active: this.active,
        run: this.run,
        enable: this.enable,
        select: this.select,
        id: this.id
      };
    }
  }, {
    key: "renderTool",
    value: function renderTool(view) {
      if (isEmpty(view)) return null;
      return this._isDisplayed ? /*#__PURE__*/React.createElement(Button, {
        item: this.toJSON(),
        key: v4(),
        view: view
      }) : null;
    }
  }, {
    key: "displayTool",
    value: function displayTool() {
      this._isDisplayed = true;
    }
  }, {
    key: "hideTool",
    value: function hideTool() {
      this._isDisplayed = false;
    }
  }, {
    key: "isDisplayed",
    value: function isDisplayed() {
      return this._isDisplayed;
    }
  }, {
    key: "displayInToolGroup",
    value: function displayInToolGroup() {
      this._isHiddenInToolGroup = false;
    }
  }, {
    key: "hideInToolGroup",
    value: function hideInToolGroup() {
      this._isHiddenInToolGroup = true;
    }
  }, {
    key: "isIntoMoreSection",
    value: function isIntoMoreSection() {
      return this._isHiddenInToolGroup;
    }
  }, {
    key: "run",
    get: function get() {
      return true;
    }
  }, {
    key: "enable",
    get: function get() {
      return function () {
        return true;
      };
    }
  }, {
    key: "active",
    get: function get() {
      return function () {
        return false;
      };
    }
  }], [{
    key: "id",
    get: function get() {
      return this.name;
    }
  }]);

  return Tools;
}(), _temp$a)) || _class$a);

var QuestionsNodeView = /*#__PURE__*/function (_AbstractNodeView) {
  _inherits(QuestionsNodeView, _AbstractNodeView);

  var _super = _createSuper(QuestionsNodeView);

  function QuestionsNodeView(node, view, getPos, decorations, createPortal, Component, context) {
    var _this;

    _classCallCheck(this, QuestionsNodeView);

    _this = _super.call(this, node, view, getPos, decorations, createPortal, Component, context);
    _this.node = node;
    _this.outerView = view;
    _this.getPos = getPos;
    _this.context = context;
    return _this;
  }

  _createClass(QuestionsNodeView, [{
    key: "update",
    value: function update(node) {
      // if (!node.sameMarkup(this.node)) return false;
      this.node = node;

      if (this.context.pmViews[node.attrs.id]) {
        var state = this.context.pmViews[node.attrs.id].state;
        var start = node.content.findDiffStart(state.doc.content);

        if (start != null) {
          var _node$content$findDif = node.content.findDiffEnd(state.doc.content),
              endA = _node$content$findDif.a,
              endB = _node$content$findDif.b;

          var overlap = start - Math.min(endA, endB);

          if (overlap > 0) {
            endA += overlap;
            endB += overlap;
          }

          this.context.pmViews[node.attrs.id].dispatch(state.tr.replace(start, endB, node.slice(start, endA)).setMeta('fromOutside', true));
        }
      }

      return true;
    }
  }]);

  return QuestionsNodeView;
}(AbstractNodeView);

function _templateObject$6() {
  var data = _taggedTemplateLiteral(["\n  border-bottom: 1px solid #d9d9d9;\n  color: #979797;\n  font-family: 'Fira Sans';\n  font-size: 15px;\n  font-weight: bold;\n  letter-spacing: 0.6px;\n  list-style: none;\n  line-height: 0;\n  margin: 5px 0;\n  display: block;\n  padding: 20px 0 10px;\n  margin-left: 10px;\n  text-align: left;\n  text-transform: uppercase;\n  width: 51%;\n"]);

  _templateObject$6 = function _templateObject() {
    return data;
  };

  return data;
}
var LeftMenuStyled = styled.div(_templateObject$6());

var LeftMenuTitle = function LeftMenuTitle(_ref) {
  var title = _ref.title;

  var _useTranslation = useTranslation(),
      t = _useTranslation.t,
      i18n = _useTranslation.i18n;

  return /*#__PURE__*/React.createElement(LeftMenuStyled, null, !isEmpty(i18n) && i18n.exists("Wax.Various.".concat(title)) ? t("Wax.Various.".concat(title)) : title);
};

function _templateObject2$3() {
  var data = _taggedTemplateLiteral(["\n  ", "\n"]);

  _templateObject2$3 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$7() {
  var data = _taggedTemplateLiteral(["\n  pointer-events: none;\n"]);

  _templateObject$7 = function _templateObject() {
    return data;
  };

  return data;
}
var activeStyles$1 = css(_templateObject$7());
var StyledButton = styled(MenuButton)(_templateObject2$3(), function (props) {
  return props.active && activeStyles$1;
});

var LeftSideButton = function LeftSideButton(_ref) {
  var _ref$view = _ref.view,
      view = _ref$view === void 0 ? {} : _ref$view,
      item = _ref.item;

  var _useTranslation = useTranslation(),
      t = _useTranslation.t,
      i18n = _useTranslation.i18n;

  var active = item.active,
      icon = item.icon,
      label = item.label,
      run = item.run,
      select = item.select,
      title = item.title;

  var _useContext = useContext(WaxContext),
      main = _useContext.pmViews.main,
      activeViewId = _useContext.activeViewId,
      activeView = _useContext.activeView;

  var isEditable = main.props.editable(function (editable) {
    return editable;
  });
  var dispatch = view.dispatch,
      state = view.state;

  var handleMouseDown = function handleMouseDown(e, editorState) {
    e.preventDefault();
    run(editorState, dispatch);
  }; // const isActive = !!(
  //   active(state, activeViewId) && select(state, activeViewId)
  // );


  var isActive = !!active(state, activeViewId);
  var isDisabled = !select(state, activeViewId, activeView);
  if (!isEditable) isDisabled = true;
  var LeftSideButtonComponent = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledButton, {
      active: isActive || false,
      disabled: isDisabled,
      iconName: icon,
      label: !isEmpty(i18n) && i18n.exists("Wax.BlockLevel.".concat(label)) ? t("Wax.BlockLevel.".concat(label)) : label,
      onMouseDown: function onMouseDown(e) {
        return handleMouseDown(e, view.state, view.dispatch);
      },
      title: !isEmpty(i18n) && i18n.exists("Wax.BlockLevel.".concat(title)) ? t("Wax.BlockLevel.".concat(title)) : title
    });
  }, [isActive, isDisabled, t("Wax.BlockLevel.".concat(title))]);
  return LeftSideButtonComponent;
};

function _templateObject$8() {
  var data = _taggedTemplateLiteral(["\n  .Dropdown-root {\n    position: relative;\n  }\n\n  .Dropdown-control {\n    position: relative;\n    overflow: hidden;\n    background-color: white;\n    border: 1px solid #ccc;\n    border-radius: 2px;\n    box-sizing: border-box;\n    color: #333;\n    cursor: default;\n    outline: none;\n    padding: 8px 52px 8px 10px;\n    transition: all 200ms ease;\n  }\n\n  .Dropdown-control:hover {\n    box-shadow: 0 1px 0 rgba(0, 0, 0, 0.06);\n  }\n\n  .Dropdown-arrow {\n    border-color: #999 transparent transparent;\n    border-style: solid;\n    border-width: 5px 5px 0;\n    content: ' ';\n    display: block;\n    height: 0;\n    margin-top: -ceil(2.5);\n    position: absolute;\n    right: 10px;\n    top: 14px;\n    width: 0;\n  }\n\n  .is-open .Dropdown-arrow {\n    border-color: transparent transparent #999;\n    border-width: 0 5px 5px;\n  }\n\n  .Dropdown-menu {\n    background-color: white;\n    border: 1px solid #ccc;\n    box-shadow: 0 1px 0 rgba(0, 0, 0, 0.06);\n    box-sizing: border-box;\n    margin-top: -1px;\n    max-height: 200px;\n    overflow-y: auto;\n    position: absolute;\n    top: 100%;\n    width: 100%;\n    z-index: 1000;\n    -webkit-overflow-scrolling: touch;\n  }\n\n  .Dropdown-menu .Dropdown-group > .Dropdown-title {\n    padding: 8px 10px;\n    color: rgba(51, 51, 51, 1);\n    font-weight: bold;\n    text-transform: capitalize;\n  }\n\n  .Dropdown-option {\n    box-sizing: border-box;\n    color: rgba(51, 51, 51, 0.8);\n    cursor: pointer;\n    display: block;\n    padding: 8px 10px;\n  }\n\n  .Dropdown-option:last-child {\n    border-bottom-right-radius: 2px;\n    border-bottom-left-radius: 2px;\n  }\n\n  .Dropdown-option:hover {\n    background-color: #f2f9fc;\n    color: #333;\n  }\n\n  .Dropdown-option.is-selected {\n    background-color: #f2f9fc;\n    color: #333;\n  }\n\n  .Dropdown-noresults {\n    box-sizing: border-box;\n    color: #ccc;\n    cursor: default;\n    display: block;\n    padding: 8px 10px;\n  }\n"]);

  _templateObject$8 = function _templateObject() {
    return data;
  };

  return data;
}
var ReactDropDownStyles = css(_templateObject$8());

var getDuration = function getDuration(timestamp) {
  var today = moment();
  var stamp = moment(timestamp);
  return moment.duration(today.diff(stamp));
};

var DateParser = function DateParser(props) {
  var children = props.children,
      timestamp = props.timestamp,
      dateFormat = props.dateFormat,
      humanizeThreshold = props.humanizeThreshold;

  var renderTimeAgo = function renderTimeAgo() {
    if (!timestamp) return '';
    var duration = getDuration(timestamp);
    return duration.humanize();
  };

  var renderTimestamp = function renderTimestamp() {
    if (!timestamp) return '';
    var duration = getDuration(timestamp);

    if (duration.asDays() < humanizeThreshold) {
      return "".concat(duration.humanize(), " ago");
    }

    return moment(timestamp).format(dateFormat);
  };

  return children(renderTimestamp(), renderTimeAgo());
};

DateParser.propTypes = {
  /** The date string. Can be any date parsable by momentjs. */
  timestamp: propTypes.oneOfType([propTypes.string, propTypes.number, Date]).isRequired,

  /** Format of the rendered date. */
  dateFormat: propTypes.string,

  /** Humanize duration threshold */
  humanizeThreshold: propTypes.number
};

var useDebounce = function useDebounce(value, delay) {
  var _useState = useState(value),
      _useState2 = _slicedToArray(_useState, 2),
      debouncedValue = _useState2[0],
      setDebouncedValue = _useState2[1];

  useEffect(function () {
    var handler = setTimeout(function () {
      setDebouncedValue(value);
    }, delay);
    return function () {
      clearTimeout(handler);
    };
  }, [value]);
  return debouncedValue;
};

var useOnClickOutside = function useOnClickOutside(ref, handler) {
  useEffect(function () {
    var listener = function listener(event) {
      /* Do nothing if clicking ref's element or descendent elements */
      if (!ref.current || ref.current.contains(event.target)) {
        return;
      }

      handler(event);
    };

    document.addEventListener('mousedown', listener);
    document.addEventListener('touchstart', listener);
    return function () {
      document.removeEventListener('mousedown', listener);
      document.removeEventListener('touchstart', listener);
    };
  }, [ref, handler]);
};

/**
 * Only works with SVG icons from icons.js
 */

var SVGIcon = function SVGIcon(props) {
  var className = props.className,
      name = props.name;
  var Component = icons[name];
  return /*#__PURE__*/React.createElement(Component, {
    className: className
  });
};

function _templateObject8() {
  var data = _taggedTemplateLiteral(["\n  background: #bfc4cd;\n  border-radius: 4px;\n  height: 22px;\n  position: relative;\n  right: 3px;\n  top: 3px;\n  width: 22px;\n  z-index: 999;\n"]);

  _templateObject8 = function _templateObject8() {
    return data;
  };

  return data;
}

function _templateObject7() {
  var data = _taggedTemplateLiteral(["\n  display: flex;\n"]);

  _templateObject7 = function _templateObject7() {
    return data;
  };

  return data;
}

function _templateObject6() {
  var data = _taggedTemplateLiteral(["\n  align-items: flex-start;\n  display: flex;\n  flex-direction: column;\n\n  button {\n    border-radius: 4px;\n    left: -33px;\n    margin-left: 4px;\n    padding-left: 25px;\n    position: relative;\n  }\n"]);

  _templateObject6 = function _templateObject6() {
    return data;
  };

  return data;
}

function _templateObject5() {
  var data = _taggedTemplateLiteral(["\n  align-self: flex-end;\n  background: none;\n  border: none;\n  color: #0042c7;\n  cursor: pointer;\n"]);

  _templateObject5 = function _templateObject5() {
    return data;
  };

  return data;
}

function _templateObject4$1() {
  var data = _taggedTemplateLiteral(["\n  display: flex;\n  width: 150px;\n"]);

  _templateObject4$1 = function _templateObject4() {
    return data;
  };

  return data;
}

function _templateObject3$1() {
  var data = _taggedTemplateLiteral(["\n  border: none;\n  border-bottom: 1px solid grey;\n  font-size: 14px;\n  font-weight: 400;\n  width: 150px;\n\n  &:focus {\n    outline: none;\n  }\n"]);

  _templateObject3$1 = function _templateObject3() {
    return data;
  };

  return data;
}

function _templateObject2$4() {
  var data = _taggedTemplateLiteral(["\n  ", "\n"]);

  _templateObject2$4 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$9() {
  var data = _taggedTemplateLiteral(["\n  pointer-events: none;\n"]);

  _templateObject$9 = function _templateObject() {
    return data;
  };

  return data;
}
var activeStyles$2 = css(_templateObject$9());
var StyledButton$1 = styled(MenuButton)(_templateObject2$4(), function (props) {
  return props.active && activeStyles$2;
});
var Input = styled.input(_templateObject3$1());
var FlexDiv = styled.div(_templateObject4$1());
var Add = styled.button(_templateObject5());
var CustomTagList = styled.div(_templateObject6());
var TagBoxWrapper = styled.div(_templateObject7());
var Box = styled.div(_templateObject8());
var initialArr = [];

var CustomTagBlockComponent = function CustomTagBlockComponent(_ref) {
  var isShowTag = _ref.isShowTag,
      item = _ref.item;

  var _useTranslation = useTranslation(),
      t = _useTranslation.t,
      i18n = _useTranslation.i18n;

  var ref = useRef();

  var _useState = useState(''),
      _useState2 = _slicedToArray(_useState, 2),
      inputValue = _useState2[0],
      setInputValue = _useState2[1];

  var _useContext = useContext(ApplicationContext),
      app = _useContext.app;

  var _useContext2 = useContext(WaxContext),
      main = _useContext2.pmViews.main,
      activeView = _useContext2.activeView,
      activeViewId = _useContext2.activeViewId;

  var state = main.state;
  var $from = state.selection.$from;
  var className = $from.parent.attrs["class"] ? $from.parent.attrs["class"] : '';
  var customTagsConfig = app.config.get('config.CustomTagService');
  var configTags = customTagsConfig && customTagsConfig.tags ? customTagsConfig.tags : initialArr;
  var saveTags = customTagsConfig && customTagsConfig.updateTags ? customTagsConfig.updateTags : function () {
    return true;
  };

  var _useState3 = useState(configTags),
      _useState4 = _slicedToArray(_useState3, 2),
      allTags = _useState4[0],
      setAllTags = _useState4[1];

  var tagStatus = item.active(activeView.state, activeViewId, className, allTags);
  var isEditable = main.props.editable(function (editable) {
    return editable;
  });
  var isDisabled = !item.select(state, activeViewId, activeView);
  if (!isEditable) isDisabled = true;
  useEffect(function () {
    if (ref.current) ref.current.focus();
  }, [isShowTag]);

  var onChangeTagName = function onChangeTagName() {
    setInputValue(ref.current.value);
  };

  var onClickAdd = function onClickAdd() {
    if (inputValue.trim() === '') return;
    configTags.push({
      label: inputValue,
      tagType: 'block'
    });
    setAllTags(configTags);
    saveTags({
      label: inputValue,
      tagType: 'block'
    });
    setInputValue('');
    if (ref.current) ref.current.focus();
  };

  var handleKeyDown = function handleKeyDown(event) {
    if (event.key === 'Enter' || event.which === 13) {
      onClickAdd();
    }
  };

  var renderTagList = function renderTagList() {
    var tagList = [];
    var blockTags = allTags.filter(function (tag) {
      return tag.tagType === 'block';
    });
    blockTags.forEach(function (blockTag) {
      tagList.push( /*#__PURE__*/React.createElement(TagBoxWrapper, {
        key: v4()
      }, /*#__PURE__*/React.createElement(Box, {
        key: v4()
      }), /*#__PURE__*/React.createElement(StyledButton$1, {
        active: tagStatus[blockTag.label],
        disabled: isDisabled,
        key: v4(),
        label: blockTag.label,
        onMouseDown: function onMouseDown() {
          item.run(activeView.state, activeView.dispatch, blockTag.label);
        },
        title: blockTag.label
      })));
    });
    return /*#__PURE__*/React.createElement(CustomTagList, null, tagList);
  };

  return useMemo(function () {
    return /*#__PURE__*/React.createElement(React.Fragment, null, isShowTag && /*#__PURE__*/React.createElement(FlexDiv, null, /*#__PURE__*/React.createElement(Input, {
      onChange: onChangeTagName,
      onKeyPress: handleKeyDown,
      ref: ref,
      type: "text",
      value: inputValue
    }), /*#__PURE__*/React.createElement(Add, {
      onClick: onClickAdd
    }, !isEmpty(i18n) && i18n.exists("Wax.Various.Add") ? t("Wax.Various.Add") : 'Add')), renderTagList());
  }, [isShowTag, inputValue, JSON.stringify(tagStatus), isDisabled, t("Wax.Various.Add")]);
};

function _templateObject5$1() {
  var data = _taggedTemplateLiteral(["\n  border-radius: 4px;\n  left: -33px;\n  margin-left: 4px;\n  padding-left: 25px;\n  position: relative;\n\n  ", "\n"]);

  _templateObject5$1 = function _templateObject5() {
    return data;
  };

  return data;
}

function _templateObject4$2() {
  var data = _taggedTemplateLiteral(["\n  background: #bfc4cd;\n  border-radius: 4px;\n  height: 22px;\n  position: relative;\n  right: 3px;\n  top: 3px;\n  width: 22px;\n  z-index: 999;\n"]);

  _templateObject4$2 = function _templateObject4() {
    return data;
  };

  return data;
}

function _templateObject3$2() {
  var data = _taggedTemplateLiteral(["\n  display: flex;\n  margin-bottom: 4px;\n"]);

  _templateObject3$2 = function _templateObject3() {
    return data;
  };

  return data;
}

function _templateObject2$5() {
  var data = _taggedTemplateLiteral(["\n  font-size: 14px;\n  margin-bottom: 4px;\n  text-transform: uppercase;\n"]);

  _templateObject2$5 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$a() {
  var data = _taggedTemplateLiteral(["\n  pointer-events: none;\n"]);

  _templateObject$a = function _templateObject() {
    return data;
  };

  return data;
}
var activeStyles$3 = css(_templateObject$a());
var GroupName = styled.div(_templateObject2$5());
var OENToolWrapper = styled.div(_templateObject3$2());
var Box$1 = styled.div(_templateObject4$2());
var StyledButton$2 = styled(MenuButton)(_templateObject5$1(), function (props) {
  return props.active && activeStyles$3;
});

var OENToolGroup = function OENToolGroup(_ref) {
  var item = _ref.item;

  var _useContext = useContext(ApplicationContext),
      app = _useContext.app;

  var _useContext2 = useContext(WaxContext),
      main = _useContext2.pmViews.main,
      activeView = _useContext2.activeView,
      activeViewId = _useContext2.activeViewId;

  var state = main.state;
  var isEditable = main.props.editable(function (editable) {
    return editable;
  });
  var isDisabled = !item.select(state, activeViewId, activeView);
  if (!isEditable) isDisabled = true;
  var OENToolsConfig = app.config.get('config.OENContainersService');
  var containersActive = item.active(main.state, OENToolsConfig);
  return useMemo(function () {
    return /*#__PURE__*/React.createElement(React.Fragment, null, OENToolsConfig.map(function (groupTool) {
      return /*#__PURE__*/React.createElement("div", {
        key: v4()
      }, /*#__PURE__*/React.createElement(GroupName, {
        key: v4()
      }, groupTool.groupHeader, " "), groupTool.items.map(function (tool) {
        return /*#__PURE__*/React.createElement(OENToolWrapper, {
          key: v4()
        }, /*#__PURE__*/React.createElement(Box$1, {
          key: v4()
        }), /*#__PURE__*/React.createElement(StyledButton$2, {
          active: containersActive[tool.className] || false,
          disabled: isDisabled,
          key: v4(),
          label: tool.displayName,
          onMouseDown: function onMouseDown() {
            var _main$state$selection = main.state.selection,
                from = _main$state$selection.from,
                to = _main$state$selection.to;
            var isInOenContainer = false;
            main.state.doc.nodesBetween(from, to, function (node, pos) {
              if (node.type.name === 'oen_container' || node.type.name === 'oen_aside') {
                isInOenContainer = true;
              }
            });

            if (isInOenContainer) {
              var range = main.state.selection.$from.blockRange(main.state.selection.$to);
              var target = range && liftTarget(range);
              if (target == null) return false;
              main.dispatch(main.state.tr.lift(range, target));
            }

            var node = tool.isSection ? 'oen_section' : 'oen_container';
            wrapIn(main.state.config.schema.nodes[node], {
              "class": tool.className,
              id: v4()
            })(main.state, main.dispatch);
            setTimeout(function () {
              main.focus();
            });
          },
          title: tool.displayName
        }));
      }));
    }));
  }, [isDisabled, containersActive]);
};

function _templateObject2$6() {
  var data = _taggedTemplateLiteral(["\n  background: #bfc4cd;\n  border-radius: 4px;\n  height: 22px;\n  position: relative;\n  right: 3px;\n  top: 3px;\n  width: 22px;\n  z-index: 999;\n"]);

  _templateObject2$6 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$b() {
  var data = _taggedTemplateLiteral(["\n  display: flex;\n\n  button {\n    border-radius: 4px;\n    left: -33px;\n    margin-left: 4px;\n    padding-left: 25px;\n    position: relative;\n  }\n"]);

  _templateObject$b = function _templateObject() {
    return data;
  };

  return data;
}
var Wrapper$3 = styled.div(_templateObject$b());
var Box$2 = styled.div(_templateObject2$6());

var BlockElement = function BlockElement(props) {
  var item = props.item,
      onClick = props.onClick,
      view = props.view;
  return /*#__PURE__*/React.createElement(Wrapper$3, {
    onClick: onClick
  }, /*#__PURE__*/React.createElement(Box$2, null), item.renderTool(view));
};

function _templateObject4$3() {
  var data = _taggedTemplateLiteral(["\n  cursor: pointer;\n  fill: ", ";\n  height: 14px;\n  width: 14px;\n"]);

  _templateObject4$3 = function _templateObject4() {
    return data;
  };

  return data;
}

function _templateObject3$3() {
  var data = _taggedTemplateLiteral(["\n  display: flex;\n  justify-content: space-between;\n"]);

  _templateObject3$3 = function _templateObject3() {
    return data;
  };

  return data;
}

function _templateObject2$7() {
  var data = _taggedTemplateLiteral(["\n  > div:not(:last-child) {\n    margin-bottom: 4px;\n  }\n"]);

  _templateObject2$7 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$c() {
  var data = _taggedTemplateLiteral(["\n  font-size: 14px;\n  margin-bottom: 4px;\n  text-transform: uppercase;\n"]);

  _templateObject$c = function _templateObject() {
    return data;
  };

  return data;
}
var GroupName$1 = styled.div(_templateObject$c());
var ListWrapper = styled.div(_templateObject2$7());
var FlexDiv$1 = styled.div(_templateObject3$3());
var StyledIcon = styled(SVGIcon)(_templateObject4$3(), th('colorPrimary'));

var BlockElementGroup = function BlockElementGroup(_ref) {
  var groupName = _ref.groupName,
      items = _ref.items,
      view = _ref.view;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      isIconClicked = _useState2[0],
      setIconClicked = _useState2[1];

  var onIconClick = function onIconClick() {
    setIconClicked(!isIconClicked);
  };

  var _useTranslation = useTranslation(),
      t = _useTranslation.t,
      i18n = _useTranslation.i18n;

  var translatedLabel = function translatedLabel(translation, defaultLabel) {
    return !isEmpty(i18n) && i18n.exists(translation) ? t(translation) : defaultLabel;
  };

  return /*#__PURE__*/React.createElement(React.Fragment, null, groupName !== 'Custom Block' && groupName !== 'OEN Containers' && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(GroupName$1, null, translatedLabel("Wax.Various.".concat(groupName), groupName)), /*#__PURE__*/React.createElement(ListWrapper, null, items && items.map(function (item) {
    return /*#__PURE__*/React.createElement(BlockElement, {
      item: item,
      key: item.name,
      view: view
    });
  }))), groupName === 'OEN Containers' && /*#__PURE__*/React.createElement(OENToolGroup, {
    item: items[0],
    key: "oen",
    view: view
  }), groupName === 'Custom Block' && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(FlexDiv$1, null, /*#__PURE__*/React.createElement(GroupName$1, null, translatedLabel("Wax.Various.".concat(groupName), groupName)), /*#__PURE__*/React.createElement("div", {
    "aria-hidden": "true",
    onClick: onIconClick
  }, !isIconClicked && /*#__PURE__*/React.createElement(StyledIcon, {
    name: "IconCross"
  }), isIconClicked && /*#__PURE__*/React.createElement(StyledIcon, {
    name: "IconMinus"
  }))), /*#__PURE__*/React.createElement(ListWrapper, null, /*#__PURE__*/React.createElement(CustomTagBlockComponent, {
    isShowTag: isIconClicked,
    item: items[0],
    view: view
  }))));
};

function _templateObject$d() {
  var data = _taggedTemplateLiteral(["\n  height: 100%;\n  overflow-y: auto;\n  padding: 8px 8px 16px 8px;\n  > div:not(:last-child) {\n    margin-bottom: 10px;\n  }\n"]);

  _templateObject$d = function _templateObject() {
    return data;
  };

  return data;
}
var Wrapper$4 = styled.div(_templateObject$d());

var BlockLevelTools = function BlockLevelTools(props) {
  var groups = props.groups,
      view = props.view;
  return /*#__PURE__*/React.createElement(Wrapper$4, null, groups && groups.map(function (group) {
    return /*#__PURE__*/React.createElement(BlockElementGroup, {
      groupName: group.groupName,
      items: group.items,
      key: group.groupName,
      view: view
    });
  }));
};

function _templateObject6$1() {
  var data = _taggedTemplateLiteral(["\n  background: #fff;\n  height: 100%;\n  width: 100%;\n"]);

  _templateObject6$1 = function _templateObject6() {
    return data;
  };

  return data;
}

function _templateObject5$2() {
  var data = _taggedTemplateLiteral(["\n  cursor: pointer;\n  margin: 0 4px 4px 4px;\n  ", "\n  ", "\n\n  padding: 8px;\n\n  &:first-child {\n    margin-top: 4px;\n  }\n\n  &:hover {\n    background: ", ";\n  }\n"]);

  _templateObject5$2 = function _templateObject5() {
    return data;
  };

  return data;
}

function _templateObject4$4() {
  var data = _taggedTemplateLiteral(["\n  display: none;\n  cursor: not-allowed;\n  opacity: 0.4;\n"]);

  _templateObject4$4 = function _templateObject4() {
    return data;
  };

  return data;
}

function _templateObject3$4() {
  var data = _taggedTemplateLiteral(["\n  background: ", ";\n  box-shadow: 0 0 1px ", ";\n"]);

  _templateObject3$4 = function _templateObject3() {
    return data;
  };

  return data;
}

function _templateObject2$8() {
  var data = _taggedTemplateLiteral(["\n  background: #fff;\n  display: flex;\n  flex-direction: column;\n"]);

  _templateObject2$8 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$e() {
  var data = _taggedTemplateLiteral(["\n  display: flex;\n  height: 100%;\n"]);

  _templateObject$e = function _templateObject() {
    return data;
  };

  return data;
}
var Wrapper$5 = styled.div(_templateObject$e());
var Tabs = styled.div(_templateObject2$8());
var activeTab = css(_templateObject3$4(), th('colorBackgroundTabs'), th('colorPrimary'));
var disabledTab = css(_templateObject4$4());
var Tab = styled.div(_templateObject5$2(), function (props) {
  return props.active && activeTab;
}, function (props) {
  return props.disabled && disabledTab;
}, th('colorBackgroundTabs'));
var Content = styled.div(_templateObject6$1());

var TabsPane = function TabsPane(props) {
  var tabList = props.tabList;
  if (!tabList || tabList.length === 0) return null;

  var _useState = useState(tabList[0].id),
      _useState2 = _slicedToArray(_useState, 2),
      tabDisplay = _useState2[0],
      setTabDisplay = _useState2[1];

  return /*#__PURE__*/React.createElement(Wrapper$5, null, /*#__PURE__*/React.createElement(Tabs, null, tabList.map(function (tab) {
    return /*#__PURE__*/React.createElement(Tab, {
      active: tabDisplay === tab.id,
      disabled: tab.disabled,
      key: tab.id,
      onClick: function onClick() {
        if (!tab.disabled) setTabDisplay(tab.id);
      },
      title: tab.title
    }, /*#__PURE__*/React.createElement(SVGIcon, {
      name: tab.icon
    }));
  })), /*#__PURE__*/React.createElement(Content, null, tabList.find(function (tab) {
    return tabDisplay === tab.id;
  }).component));
};

export { ApplicationContext, BlockLevelTools, Button, Commands, ComponentPlugin, DateParser, DefaultSchema, DocumentHelpers, EditoriaSchema, FakeCursorPlugin, SVGIcon as Icon, LayoutService, LeftMenuTitle, LeftSideButton, MenuButton, MenuService, Middleware, OverlayService, PortalContext, PortalService, QuestionsNodeView, ReactDropDownStyles, RulesService, SchemaHelpers, SchemaService, Service, ShortCutsService, StateContext, TabsPane as Tabs, ToolGroup, ToolGroupComponent, ToolGroups, Tools, Wax, WaxContext, WaxView, icons, trackedTransaction, useDebounce, useInjection, useOnClickOutside };
